/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: conv.c,v 1.4 2000/01/04 06:40:54 tmurakam Exp $
 */
/*
 * conv.c 
 */

#include <stdio.h>

#include "type.h"
#include "conv.h"
#include "util.h"

#define	EQ2NPL	10001750.0	/* Distance from Equator to each pole */
#define	MAXEU	0x1000000	/* Maximum value of longtitude */

/* unit conversion */
double lat_int2lat(lat_int_t lat_int)
{
	double lat;

	if (lat_int & 0x00800000) {
		lat_int |= 0xff000000;	/* sign expansion */
	}
	lat = (double)lat_int * 2.0 / EQ2NPL * 90.0;
	if (lat > 90.0) {
		lat -= 180.0;
	}

	return lat;
}

double lat_dec2lat(lat_dec_t lat_dec)
{
	double lat;

	lat = (double)lat_dec / 128.0 /  EQ2NPL * 90.0;
	if (lat > 90.0) {
		lat -= 180.0;
	}

	return lat;
}

double lat_long2lat(lat_long_t lat_long)
{
	double lat;

	lat = (double)lat_long /  EQ2NPL * 90.0;
	if (lat > 90.0) {
		lat -= 180.0;
	}

	return lat;
}

double lat_micro2lat(lat_micro_t *lat_micro)
{
	double lat;

	if (lat_micro->long_part < 0) {
			lat = ~(u_int32)lat_micro->long_part + (~(u_int16)lat_micro->dec_part + 1) / 65536.0;
			lat = -lat;
	} else {
		lat = (double)lat_micro->long_part + (double)lat_micro->dec_part / 65536.0;
	}
	
	lat = lat / EQ2NPL * 90.0;
	if (lat > 90.0) {
		lat -= 180.0;
	}
	return lat;
}

double lon_int2lon(lon_int_t lon_int)
{
	double lon;

	if (lon_int & 0x00800000) {
		lon_int |= 0xff000000;	/* sign expansion */
	}
	lon = (double)lon_int / (double)MAXEU * 360.0;
	if (lon > 180.0) {
		lon -= 360.0;
	}
	return lon;
}

double lon_dec2lon(lon_dec_t lon_dec)
{
	double lon;

	lon = (double)lon_dec / 256.0 / (double)MAXEU * 360.0;
	if (lon > 180.0) {
		lon -= 360.0;
	}
	return lon;
}

double lon_micro2lon(lon_micro_t *lon_micro)
{
	double lon;

	if (lon_micro->long_part < 0) {
			lon = ~(u_int32)lon_micro->long_part + ((double)(~(u_int16)lon_micro->dec_part) + 1.0) / 65536.0;
			lon = -lon;
	} else {
		lon = (double)lon_micro->long_part + (double)lon_micro->dec_part / 65536.0;
	}
	lon = lon / 256.0 / (double)MAXEU * 360.0;
	if (lon > 180.0) {
		lon -= 360.0;
	}
	return lon;
}

double alt_micro2alt(alt_micro_t *alt_micro)
{
	double alt;

	if (alt_micro->long_part < 0) {
		alt = ~(u_int32)alt_micro->long_part + ((double)(~(u_int16)alt_micro->dec_part) + 1.0) / 65536.0;
		alt = -alt;
	} else {
		alt = (double)alt_micro->long_part + (double)alt_micro->dec_part / 65536.0;
	}
	return alt;
}

double deg162deg(u_int16 deg16)
{
	double deg = (double)deg16 * 360.0 / 65536.0;
	return deg;
}

double deg322deg(u_int32 deg32)
{
	double deg = (double)deg32 * 360.0 / 65536.0 / 65536.0;
	return deg;
}

/* string buffer */
char latlonstrbuf[LLBLEN];

/* convert lat/long value to string */
char *
latstr(double deg, char *str)
{
	int dd, mm;
	double ss;

	if (str == NULL) {
		str = latlonstrbuf;
	}

	if (deg > 0) {
		*str = 'N';
	} else {
		*str = 'S';
		deg = -deg;
	}

	dd = (int)deg;
	deg = (deg - dd) * 60;
	mm = (int)deg;
	ss = (deg - mm) * 60;

	sprintf(str + 1, "%d:%d:%.2f", dd, mm, ss);

	return str;
}

char *
lonstr(double deg, char *str)
{
	int dd, mm;
	double ss;

	if (str == NULL) {
		str = latlonstrbuf;
	}

	if (deg > 0) {
		*str = 'E';
	} else {
		*str = 'W';
		deg = -deg;
	}

	dd = (int)deg;
	deg = (deg - dd) * 60;
	mm = (int)deg;
	ss = (deg - mm) * 60;
	sprintf(str + 1, "%d:%d:%.2f", dd, mm, ss);

	return str;
}

char *
RunwayName(u_int8 id)
{
	static char name[10];
	int num;

	num = id & 0x3f;
	switch (id & 0xc0) {
	case 0xc0:
		sprintf(name, "%dC", num);
		break;
	case 0x80:
		sprintf(name, "%dR", num);
		break;
	case 0x40:
		sprintf(name, "%dL", num);
		break;
	case 0x0:
		sprintf(name, "%d", num);
		break;
	}
	return name;
}
