/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: label.c,v 1.6 2000/01/04 06:41:04 tmurakam Exp $
 */
/*
 * label.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef	WIN32
#include <direct.h>
#endif
#include <errno.h>

#include "type.h"
#include "label.h"
#include "util.h"

#define	HASHSIZE	4095
#define	HASHKEY(x)	(x % HASHSIZE)	/* = x % HASHSIZE */

typedef struct labeldata {
	u_int32			label;	/* pointer */
	unsigned int		seq;	/* sequence number */
	struct labeldata	*next;
} Label;

int fSeqLabel = 0;
unsigned int LabelSeq = 1;

static Label *LabelHash[HASHSIZE];


/* clear all labels */
extern void
ClearLabel(void)
{
	int i;

	LabelSeq = 0;	/* ### needed ??? */
	for (i=0; i<HASHSIZE; i++) {
		LabelHash[i] = NULL;
	}
}

/* add label */
extern void
PutLabel(u_int32 point)
{
	Label	*p;
	int	hashkey;

	if (! IS_PASS1) {
		return;		/* do nothing */
	}

	/* duplicate check */
	hashkey = HASHKEY(point);
	for (p = LabelHash[hashkey]; p; p = p->next) {
		if (p->label == point) {
			/* duplicated ! */
			return;
		}
	}

	/* get label */
	p = (struct labeldata *)malloc(sizeof(struct labeldata));
	if (!p) {
		/* can't allocate label */
		fprintf(stderr, "No memory\n");
		exit(1);
	}
	p->seq = LabelSeq++;

	/* add link */
	p->label = point;
	if (!LabelHash[hashkey]) {
		p->next = 0;
		LabelHash[hashkey] = p;
	} else {
		p->next = LabelHash[hashkey];
		LabelHash[hashkey] = p;
	}
	/*fprintf(outfp, ";;; PutLabel 0x%x [%d]\n", p, labelp);*/
}


/* If there is label at current point, display it */
extern void
CheckLabel(void)
{
	Label	*p;
	u_int32	point;

	ASSERT(IS_PASS2);

	point = GetPointer();

	for (p = LabelHash[HASHKEY(point)]; p; p = p->next) {
		if (p->label == point) {
			if (fSeqLabel) {
				fprintf(outfp, ":L%d\n", p->seq);			
			} else {
				fprintf(outfp, ":L%08X\n", point);
			}
			return;
		}
	}
}

/* Display label */
extern void
PrintLabel(u_int32 point)
{
	Label *p;

	if (!outfp) return;	/* do nothing ### */

	for (p = LabelHash[HASHKEY(point)]; p; p = p->next) {
		if (p->label == point) {
			break;
		}
	}

	if (!p) {
		fprintf(stderr, "Internal error : no label(0x%x)\n", point);
		exit(1);
	}


	if (fSeqLabel) {
		fprintf(outfp, ":L%d", p->seq);
	} else {
		fprintf(outfp, ":L%08X\n", point);
	}
}

