/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: scdis.c,v 1.7 2000/01/15 07:07:12 tmurakam Exp $
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "conv.h"
#include "label.h"
#include "sec0ana.h"
#include "synthana.h"
#include "sec9ana.h"
#include "sec10ana.h"
#include "sec11ana.h"
#include "sec13ana.h"
#include "sec14ana.h"
#include "sec15ana.h"
#include "sec16ana.h"
#include "sec19ana.h"
/*#include "sec20ana.h"*/

int main(int argc, char *argv[]);
void analyze(void);
void analyzeHeader(void);
void analyzeSection(int);

struct HeaderInfo	Header;	/* header information data */

static void usage(char *argv0)
{
	fprintf(stderr, "usage: scdis [options] <bglfile> [<directory>]\n", argv0);
	fprintf(stderr, "  options:\n");
	fprintf(stderr, "     -s    : sequenced label\n");
	exit(1);
}

int
main(int argc, char *argv[])
{
	int ret = 0, i;
	char *bglfile, *outdir;

	outfp = NULL;

	fprintf(stderr, "SCDIS: MSFS Scenery Disassembler for scasm %s\n", VERSION);
	fprintf(stderr, "  Copyright (C) 1996-2000 T.Murakami\n");
	fprintf(stderr, "  E-Mail: tmurakam@mtd.biglobe.ne.jp\n");
	fprintf(stderr, "  WWW: http://www.freesc.org/\n");
	fprintf(stderr, "\n");

	if (argc < 2) {
		usage(argv[0]);
	}

	bglfile = 0;
	outdir = 0;

	for (i=1; i<argc; i++) {
		if (argv[i][0] == '-') {
			switch (argv[i][1]) {
			case 's':
				fSeqLabel = 1;
				break;

			default:
				usage(argv[0]);
			}
		}
		else {
			if (!bglfile) {
				bglfile = argv[i];
			}
			else if (!outdir) {
				outdir = argv[i];
			}
			else {
				usage(argv[0]);
			}
		}
	}

	if (!bglfile) {
		usage(argv[0]);
	}

	SetOutDir(outdir);
	OpenBglFile(bglfile);

	analyzeHeader();

	OpenSclinkControlFile();
	for (i=0; i<=21; i++) {
		analyzeSection(i);
	}

	/* print remain */
	DumpRemain();

	CloseSclinkControlFile();
	CloseBglFile();
	
	fprintf(stderr, "Done.\n\n");

	return 0;
}

/*
 * analyze BGL header
 */
void
analyzeHeader(void)
{
	int i, j;
	
	OpenOutFile(HEADER_FILE, FALSE);

	printf("Anaylizing BGL Header\n");

	Header.scenery_id = get2();
	Header.max_north = get4();	
	Header.min_north = get4();
	Header.max_east = get4();
	Header.min_east = get4();

	fprintf(outfp, ";;; Disassembled by scdis %s\n", VERSION);
	fprintf(outfp, ";;; BGL header info\n");

	fprintf(outfp, "scenery id:	0x%x\n", Header.scenery_id);
	fprintf(outfp, "max north:	%s\n", 
		latstr(lat_long2lat(Header.max_north), NULL));
	fprintf(outfp, "min north:	%s\n", 
		latstr(lat_long2lat(Header.min_north), NULL));
	fprintf(outfp, "max east:	%s\n",
		lonstr(lon_dec2lon(Header.max_east), NULL));
	fprintf(outfp, "min east:	%s\n", 
		lonstr(lon_dec2lon(Header.min_east), NULL));	

	for (i=0; i<=21; i++) {
		fprintf(outfp, "sec%d point:	", i);

		Header.sec_pointer[i] = get4();
		fprintf(outfp, "0x%x\n", Header.sec_pointer[i]);

		switch (i) {
		case 0:
			Header.min_freq = get2();
			Header.max_freq = get2();
			fprintf(outfp, "min freq:	0x%x\n", Header.min_freq);
			fprintf(outfp, "max freq:	0x%x\n", Header.max_freq);
			break;

		case 15:
			for (j=0; j<=7; j++) {
				Header.val[j] = get2();
				fprintf(outfp, "value%d:	0x%x\n", j, Header.val[j]);
			}
			break;
			
		case 21:
			Header.val[8] = get2();
			fprintf(outfp, "value8:		0x%x\n", Header.val[8]);
			break;

		default:
			break;
		}
	}

	CloseOutFile();
}

/*
 * analyze each section
 */
void
analyzeSection(int secno)
{
	if (!Header.sec_pointer[secno]) {
		printf("Analyzing section %d... does not exist\n", secno);
		return;
	}

	printf("Analyzing section %d...\n", secno);

	MovePointer(Header.sec_pointer[secno]);

	/*fprintf(fp, ";;; section %d\n", secno);*/
	switch(secno) {

		
	case 0:	/* Section 0: VOR, ILS */
		AnalyzeSec0(Header.sec_pointer[secno], 
			108.0 + Header.min_freq * 0.05,
			108.0 + Header.max_freq * 0.05);
		break;

		/* Section 1-6: Synthetic Scenery */
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
		AnalyzeSynth(Header.sec_pointer[secno], secno);
		break;
		
		/* Section 9: Procedural Scenery */
	case 9:
		AnalyzeSec9(Header.sec_pointer[secno]);
		break;

		/* Section 10: Library */
	case 10:
		AnalyzeSec10(Header.sec_pointer[secno]);
		break;

		/* Section 11: Airport */
	case 11:
		AnalyzeSec11(Header.sec_pointer[secno]);
		break;

		/* Section 13: ATIS */
	case 13:
		AnalyzeSec13(Header.sec_pointer[secno]);
		break;

		/* Section 14: NDB */
	case 14:
		AnalyzeSec14(Header.sec_pointer[secno]);
		break;

		/* sectoin 15: Dynamic scenery */
	case 15:
		AnalyzeSec15(Header.sec_pointer[secno]);
		break;

		/* sectoin 16: IMO Maker, Landme */
	case 16:
		AnalyzeSec16(Header.sec_pointer[secno]);
		break;

		/* section 19: Exception */
	case 19:
		AnalyzeSec19(Header.sec_pointer[secno]);
		break;

		/* section 20: Airport block (experimental) */
#if 0
	case 20:
		AnalyzeSec20(Header.sec_pointer[secno]);
		break;
#endif

	default:
		{
			char fname[30];
			sprintf(fname, "%s%d", SECFILENAME_PRE, secno);
			OpenOutFile(fname, TRUE);
			fprintf(outfp, ";;; sorry, bgl2scm can't decode section %d\n", secno);
			CloseOutFile();
			printf(";;; sorry, bgl2scm can't decode section %d\n", secno);
		}
		break;
	}
}


