/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec10ana.c,v 1.2 2000/01/04 06:41:23 tmurakam Exp $
 */
/*
 * Section 10 (Library) analyze
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "type.h"
#include "util.h"
#include "conv.h"
/*#include "sec10ana.h"*/
#include "sec9ana.h"

/* global variables */

/* prototypes */
static void anaSec10Main(u_int32 sec_pointer);
static void AnalyzeLibObj(u_int32 pointer);

extern void
AnalyzeSec10(u_int32 sec_pointer)
{
	char fname[30];

	sprintf(fname, "%s10", SECFILENAME_PRE);
	OpenOutFile(fname, TRUE);

	ClearLabel();
#ifndef	SKIP_PASS1
	SET_PASS1;
	anaSec10Main(sec_pointer);
#endif
	SET_PASS2;
	anaSec10Main(sec_pointer);
	ClearLabel();

	CloseOutFile();
}

static void
anaSec10Main(u_int32 sec_pointer)
{
	u_int32	offset, pos;
	u_int8	c;
	int	i;
	
	MovePointer(sec_pointer);

	while (1) {
		offset = get4();
		if (offset == 0) {
			/* EOF */
			break;
		}

		if (IS_PASS2) {
			fprintf(outfp, ";;;\n");
			fprintf(outfp, ";;; Library object header( ");
			for (i=0; i<16; i++) {
				c = get1();
				fprintf(outfp, "%02x ", c);
			}
			fprintf(outfp, ")\n");
			fprintf(outfp, ";;;\n\n");
		}
		else {
			for (i=0; i<16; i++) get1();	/* skip ID */
		}

		pos = GetPointer();

		AnalyzeLibObj(sec_pointer + offset);

		MovePointer(pos);
	}
}

static void AnalyzeLibObj(u_int32 pointer)
{
	u_int32 endpos;
	int i;
	u_int8 c;
	u_int16 size;

	MovePointer(pointer);

	if (IS_PASS2) fprintf(outfp, ";;; Library object entry ( ");

	/* id? */
	for (i=0; i<16; i++) {
		c = get1();
		if (IS_PASS2) fprintf(outfp, "%02x ", c);
	}

	/* power? */
	c = get1();
	if (IS_PASS2) fprintf(outfp, "  %02x )\n", c);

	size = get2();	/* ### */
	endpos = pointer + size;

	while (GetPointer() < endpos) {
		c = get1();

		/* check section 9 table */
		i = 0;
		while(sec9tab[i].record_id) {
			if (sec9tab[i].record_id == c) {
				(*sec9tab[i].func)(sec9tab[i].name, c, sec9tab[i].len);
				break;
			}
			i++;
		}
		if (sec9tab[i].record_id == 0) {
			if (IS_PASS2) {
				fprintf(outfp, ";;; unknown record id: 0x%x\n", c);
			}
			/*exit(1);*/
		}
	}
}	
