/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec13ana.c,v 1.3 2000/01/04 06:41:34 tmurakam Exp $
 */
/*
 * Section 13 (ATIS) analyze
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "conv.h"
#include "sec13ana.h"

/* prototypes */
void AnalyzeATIS(u_int32);
u_int16 AnalyzeATISMain(void);

extern void
AnalyzeSec13(u_int32 sec_pointer)
{
	SET_PASS2;	/* no PASS 1 */
	AnalyzeLatRangeTable(13, sec_pointer, AnalyzeATIS);
}

void AnalyzeATIS(u_int32 poitner)
{
	u_int8	c;
	u_int16	len;
	u_int32	posbak;

	while (1) {

		posbak = GetPointer();
		
		c = get1();
		if (c == 0) break;	/* end of table */
		if (c != 4) {
			fprintf(stderr, "unknown record");
			exit(1);
		}

		len = AnalyzeATISMain();
		MovePointer(posbak + len);
	}
}

/*
-- 04h ------------- ATIS (16 bytes + ATIS text)

1     2   frequency (BCD coded: 65h 28h = 128.65)
3     3   Latitude  in Iu
6     3   Longitude in Iu
9     1   Range in 2048m units
10    2   record length
12    1   preferred runway 1 (bit 6 and 7 may be set: ??)
13    1   preferred runway 2 (bit 6 and 7 may be set: ??)
14    1   preferred runway 3 (bit 6 and 7 may be set: ??)
15    1   preferred runway 4 (bit 6 and 7 may be set: ??)
16    x   Variable length 0-terminated ASCII string with the ATIS message
*/
#define	ATISMLEN	255		/* ATIS Max Message Length */
u_int16 AnalyzeATISMain(void)
{
	u_int16		freq;
	lat_int_t	lat;
	lon_int_t	lon;
	u_int8		range;
	u_int16		len;
	u_int8		pref1;
	u_int8		pref2;
	u_int8		pref3;
	u_int8		pref4;
	u_int8		name[ATISMLEN + 1], c;
	int			i;

	freq = get2();
	lat = get3();
	lon = get3();
	range = get1();
	len = get2();
	pref1 = get1();
	pref2 = get1();
	pref3 = get1();
	pref4 = get1();
	for (i=0; i<ATISMLEN; i++) {
		c = get1();
		if (c == '(') c = '{';
		if (c == ')') c = '}';
		if (c == ' ') c = '_';
		name[i] = c;
		if (c == 0) break;

		if (0x80 <= c && c <= 0x93) {
			name[i] = '%';
			i++;
			name[i] = 'A' + c - 0x80;
		}
	}
	name[ATISMLEN] = 0;	/* ad hoc */

	fprintf(outfp, "ATIS(\n");
	fprintf(outfp, "\t%d\n", range * 2);
	fprintf(outfp, "\t%s\n", latstr(lat_int2lat(lat), NULL));
	fprintf(outfp, "\t%s\n", lonstr(lon_int2lon(lon), NULL));
	fprintf(outfp, "\t1%d%d.%d%d\n", freq >> 12, (freq >> 8) & 0xf, (freq >> 4) & 0xf, freq & 0xf);
	fprintf(outfp, "\t%s ", RunwayName(pref1));
	fprintf(outfp, "%s ", RunwayName(pref2));
	fprintf(outfp, "%s ", RunwayName(pref3));
	fprintf(outfp, "%s\n", RunwayName(pref4));
	fprintf(outfp, "\t%s )\n", name);

	return len;
}

