/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec14ana.c,v 1.3 2000/01/04 06:41:40 tmurakam Exp $
 */
/*
 * Section 14 (NDB) analyze
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "conv.h"
#include "sec13ana.h"

/* prototypes */
void AnalyzeNDB(u_int32);
void AnalyzeNDBMain(void);

extern void
AnalyzeSec14(u_int32 sec_pointer)
{
	SET_PASS2;	/* no PASS 1 */
	AnalyzeLatRangeTable(14, sec_pointer, AnalyzeNDB);
}

void AnalyzeNDB(u_int32 pointer)
{
	u_int8	c;
	
	while (1) {

		c = get1();
		if (c == 0) break;	/* end of table */
		if (c != 4) {
			fprintf(stderr, "unknown record");
			exit(1);
		}

		AnalyzeNDBMain();
	}
}

/*
-- 04h ------------- NDB (42 bytes)

1     2   frequency integral part (BCD coded: 59h 03h = 359 kHz)
3     1   frequency decimal part (00h = .00 kHz; 05h = .50 kHz)
4     3   Latitude  in Iu
7     3   Longitude in Iu
10    2   Altitude in m
12    1   range in 2048m units
13    5   NDB ID; string padded with spaces
18    24  NDB Name; string padded with spaces
*/
void AnalyzeNDBMain(void)
{
	u_int16		freqint;
	u_int8		freqdec;
	lat_int_t	lat;
	lon_int_t	lon;
	u_int16		alt;
	u_int8		range;
	char		id[6], name[25];

	freqint = get2();
	freqdec = get1();
	lat = get3();
	lon = get3();
	alt = get2();
	range = get1();

	getlong(id, 5);
	id[5] = 0;
	NormalString(id);

	getlong(name, 24);
	name[24] = 0;
	NormalString(name);

	fprintf(outfp, "\tNDB( ");
	fprintf(outfp, "%d%d%d.%d%d ", (freqint >> 8) & 0xf, (freqint >> 4) & 0xf, freqint & 0xf,
		(freqdec >> 4) & 0xf, freqdec & 0xf);
	fprintf(outfp, "%d ", range * 2);
	fprintf(outfp, "%s ", id);
	fprintf(outfp, "%s ", name);
	fprintf(outfp, "%s ", latstr(lat_int2lat(lat), NULL));
	fprintf(outfp, "%s ", lonstr(lon_int2lon(lon), NULL));
	fprintf(outfp, "%d )\n", alt);
}
