/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec15ana.c,v 1.4 2000/01/04 06:41:46 tmurakam Exp $
 */
/*
 * Section 15 : Dynamic scenery analyze 
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "label.h"
#include "conv.h"
#include "sec15ana.h"

/* variables */
struct SubTable	*LatRange;	/* LatRange table */
extern SecRecordTab sec15tab[];

/* prototypes */
void AnalyzeSec15Main(u_int32);
static u_int32 anaArea15(void);
static void anaDynMain(u_int32);

extern void 
AnalyzeSec15(u_int32 sec_pointer)
{
	/* analyze LatTable */
	AnalyzeLatRangeTable(15, sec_pointer, AnalyzeSec15Main);
}

void AnalyzeSec15Main(u_int32 pointer)
{
	/* start analyze */
	ClearLabel();
	SET_PASS1;
	anaDynMain(pointer);
	SET_PASS2;
	anaDynMain(pointer);
	ClearLabel();
}

static void
anaDynMain(u_int32 basepoint)
{
	u_int8 c;
	int i;
	int in_area = 0;
	u_int32 enda = 0;
	u_int32 p;
	int endaflag = 0;

	MovePointer(basepoint);

	while (1) {

		c = get1();
		if (IS_PASS2 && endaflag) {
			fprintf(outfp, "End15\n\n");
			endaflag = 0;
		}

		/*fprintf(outfp, "%02X ", c);*/

		if (!in_area) {
			switch (c) {
			case 0x04:
				enda = anaArea15();
				in_area = 1;
				break;
				
			case 0x0:
				return;	/* End of Latrange */
				break;

			default:
				if (IS_PASS2) {
					fprintf(outfp, "; unknown record: 0x%x\n", c);
				}
				break;
			}
		}
		else {
			i = 0;

			/* for debug*/
			/*if (IS_PASS2) {
				fprintf(outfp, ";#");
				PrintLabel(GetPointer()-1);
				fprintf(outfp, "\n");
			}*/

			while(sec15tab[i].record_id) {
				if (sec15tab[i].record_id == c) {
					(*sec15tab[i].func)(sec15tab[i].name, c, sec15tab[i].len);
					break;
				}
				i++;
			}
			if (sec15tab[i].record_id == 0) {
				if (IS_PASS2) {
					fprintf(outfp, ";;; unknown record id: 0x%x\n", c);
				}
				/*exit(1);*/
			}

			/* check EndA */
			p = GetPointer();
			if (p >= enda) {
				endaflag = 1;	/* To keep the order of EndA and label ... */

				if (IS_PASS2) {

					if (p > enda) {
						fprintf(outfp, ";;;;; EndA Overrun. (now:0x%x, enda:0x%x\n", p, enda);
					}
				}
				in_area = 0;
			}
		}
	}
}

/*
 * Area15 analyze
 */
static u_int32
anaArea15(void)
{
	u_int32 basep;

	lat_dec_t	maxlat, minlat;
	lat_dec_t	bmaxlat, bminlat;
	lon_dec_t	maxlon, minlon;
	lon_dec_t	bmaxlon, bminlon;
	u_int16		offset;
	char dummy[50];

	maxlat = (s_int16)get2() * 65536;
	minlat = (s_int16)get2() * 65536;
	maxlon = (s_int16)get2() * 65536;
	minlon = (s_int16)get2() * 65536;

	offset = get2();
	basep = GetPointer();

	bmaxlat = (s_int16)get2() * 65536;
	bminlat = (s_int16)get2() * 65536;
	bmaxlon = (s_int16)get2() * 65536;
	bminlon = (s_int16)get2() * 65536;

	getlong(dummy, 24);

	/* display */
	if (IS_PASS2) {
		fprintf(outfp, "Area15( ");
		fprintf(outfp, "%s ", latstr(lat_dec2lat(maxlat), NULL));
		fprintf(outfp, "%s ", latstr(lat_dec2lat(minlat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_dec2lon(maxlon), NULL));
		fprintf(outfp, "%s\n\t", lonstr(lon_dec2lon(minlon), NULL));
		fprintf(outfp, "%s ", latstr(lat_dec2lat(bmaxlat), NULL));
		fprintf(outfp, "%s ", latstr(lat_dec2lat(bminlat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_dec2lon(bmaxlon), NULL));
		fprintf(outfp, "%s )\n", lonstr(lon_dec2lon(bminlon), NULL));
	}

	return basep + offset;
}

/*
 * unknown record (len known), or record which SCASM don't support.
 */
void
anaDynUnknown(char *name, u_int8 c, int len)
{
	if (IS_PASS2) {
		fprintf(outfp, ";;; (unknown)  %s(0x%02x) ", name, c);
	}
	if (len < 1) return; /* ad hoc */
	Dump(len - 1);
}

/*
 * dumpilen known)
 */
void
anaDynDump(char *name, u_int8 c, int len)
{
	int i;
	if (IS_PASS2) {
		fprintf(outfp, "\t; %s(0x%02x)\n", name, c);
	}
	if (len < 1) return; /* ad hoc */

	if (IS_PASS2) {
		fprintf(outfp, "\tDbx( %X ", c);
	}
	for (i=0; i<len - 1; i++) {
		c = get1();
		if (IS_PASS2) {
			fprintf(outfp, "%X ", c);
		}
	}
	if (IS_PASS2) {
		fprintf(outfp, ")\n");
	}
}

/* for instruction which have no arguments */
void anaDynNoArg(char *name, u_int8 c, int len)
{
	if (IS_PASS2) {
		fprintf(outfp, "\t%s\n", name);
	}
}

/*
- 01h ----- Rotate model to pitch attitude (5 bytes)

1     2   Pitch angle
3     2   Speed			(7fff -> 180[s])
*/
void anaDynRotate(char *name, u_int8 c, int len)
{
	s_int16	angle;
	s_int16	speed;

	angle = get2();
	speed = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %.2f %.2f )\n", name, 
			(angle * 360.0 / 65536.0), 
			(speed / 32767.0 * 180) );
	}
}

/*
-- 04h ----- Gear up/down  (2 bytes)

1     1   0 = gear up, 1 = gear down
*/
void anaDynArgBd(char *name, u_int8 c, int len)
{
	s_int8	value;

	value = get1();
	value = value ? 1 : 0;

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d )\n", name, value);
	}
}

/*
- 07h ----- Set heading (3 bytes)

1     2   New heading
*/
void anaDynArgWd(char *name, u_int8 c, int len)
{
	s_int16	value;

	value = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d )\n", name, value);
	}
}

/*
- 07h ----- Set heading (3 bytes)

1     2   New heading
*/
void anaDynHeading(char *name, u_int8 c, int len)
{
	double heading;

	heading = deg162deg(get2());

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %f )\n", name, heading);
	}
}

/*
-- 12h ----- Move East (5 bytes)

1     2   Time in seconds
3     2   East displacement in m
*/
void anaDynMove1(char *name, u_int8 c, int len)
{
	u_int16 time;
	s_int16	disp;

	time = get2();
	disp = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d )\n", name, time, disp);
	}
}

/*
-- 15h ----- Move East / North (7 bytes)

1     2   Time in seconds
3     2   East  disp. in m
5     2   North disp. in m
*/
void anaDynMove2(char *name, u_int8 c, int len)
{
	u_int16 time;
	s_int16	disp1, disp2;

	time = get2();
	disp1 = get2();
	disp2 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, time, disp1, disp2);
	}
}

/*
-- 18h ----- Move 3D (9 bytes)

1     2   Time in seconds
3     2   East  disp. in m
5     2   Alt.  disp. in m
7     2   North disp. in m
*/
void anaDynMove3(char *name, u_int8 c, int len)
{
	u_int16 time;
	s_int16	disp1, disp2, disp3;

	time = get2();
	disp1 = get2();
	disp2 = get2();
	disp3 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d %d )\n", name, time, disp1, disp2, disp3);
	}
}

/*
- 19h ----- Accelerate East (7 bytes)

1     2   Time in second
3     2   Rate
5     2   East disp. in m
*/
void anaDynAccel1(char *name, u_int8 c, int len)
{
	u_int16 time, rate;
	s_int16	disp;

	time = get2();
	rate = get2();
	disp = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %.2f %d )\n", name, time, rate / 4096.0, disp);
	}
}

/*
-- 1Ch ----- Accelerate East / North (7 bytes)

1     2   Time in second
3     2   Rate
5     2   East  disp. in m
7     2   North disp. in m
*/
void anaDynAccel2(char *name, u_int8 c, int len)
{
	u_int16 time, rate;
	s_int16	disp1, disp2;

	time = get2();
	rate = get2();
	disp1 = get2();
	disp2 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %.2f %d %d )\n", name, time, rate / 4096.0, disp1, disp2);
	}
}

/*
-- 1Fh ----- Accelerate 3D (11 bytes)

1     2   Time in second
3     2   Rate
5     2   East  disp. in m
7     2   Alt.  disp. in m
9     2   North disp. in m
*/
void anaDynAccel3(char *name, u_int8 c, int len)
{
	u_int16 time, rate;
	s_int16	disp1, disp2, disp3;

	time = get2();
	rate = get2();
	disp1 = get2();
	disp2 = get2();
	disp3 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %.2f %d %d %d )\n", 
			name, time, rate / 4096.0, disp1, disp2, disp3);
	}
}

/*
-- 20h ----- Turn left/right about point (7 bytes)

1     2   Time in seconds
3     2   Final heading
5     2   Turning radius (m)
*/
void anaDynTurn(char *name, u_int8 c, int len)
{
	u_int16 time, radius;
	double	heading;

	time = get2();
	heading = deg162deg(get2());
	radius = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %f %d )\n", name, time, heading, radius);
	}
}

/*
- 24h ----- Jump (3 bytes)

1     2   Byte offset to jump to
*/
void 
anaDynJumpCall(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;

	basep = GetPointer() - 1;

	offset = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " )\n");
	}
}


/*
-- 32h ---- Pattern declaration (8 bytes)

1     2   Model:         FE0Bh = Cessna
                         FE0Ch = Learjet
                         FE0Dh     = sailboat
                         FE0Fh     = truck
                         FE10h = large size jet
3     2   Byte offset of pattern definition
5     2   Always 00h ??		(traffic flag?)
7     1   ???				(minimum dense)
*/

void 
anaDynPattern(char *name, u_int8 c, int len)
{
	u_int32 basep;
	
	u_int16	model;
	s_int16	offset;
	u_int16	dummy;
	u_int8	flag;
	int traffic, mindense;

	basep = GetPointer() - 1;
	
	model = get2();
	offset = get2();
	dummy = get2();
	flag = get1();
	traffic = flag >> 4;
	mindense = flag & 0xf;

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " 0%X %d %d )\n", model, traffic, mindense);
	}
}


/*
-- 31h ---- Pattern setup (4 bytes)

1     1   parameter?	(number)
2     2   value?		(color code)
*/
void anaDynSetColor(char *name, u_int8 c, int len)
{
	u_int8 number, color1, color2;

	number = get1();
	color1 = get1();
	color2 = get1();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %X %X )\n", name, number, color1, color2);
	}
}

/*
-- 33h ---- Set pattern position (19 bytes)

1     2   ???
3     3   Latitude
6     2   ???
8     2   ???
10    3   Longitude
13    2   ???
15    2   Altitude above MSL in m
17    2   ???
*/
void 
anaDynSetPos(char *name, u_int8 c, int len)
{
	lat_int_t	lat_i;
	lon_int_t	lon_i;
	double		lat, lon, alt;
	u_int16		dum1, dum2, dum3, dum4;

	dum1 = get2();
	lat_i  = get3();
	lat = lat_int2lat(lat_i / 2);

	dum2 = get2();
	dum3 = get2();
	lon_i  = get3();
	lon = lon_int2lon(lon_i);

	alt = (u_int16)get2() / 65536.0;
	alt += get2();
	dum4 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		fprintf(outfp, "%s ", latstr(lat, NULL));
		fprintf(outfp, "%s ", lonstr(lon, NULL));
		fprintf(outfp, "%f )\n", alt);
	}
}

/*
-- 34h ----- LatRef

1     6   latitude?
*/
void anaDynLatRef(char *name, u_int8 c, int len)
{
	lat_micro_t	lat;

	lat.dec_part	= get2();
	lat.long_part   = get4();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %s )\n", name, latstr(lat_micro2lat(&lat), NULL));
	}
}


/**************   FS98  *****************/

/*
-- 39h ----- MoveToPos1 
*/
void anaDynMoveToPosX(char *name, u_int8 c, int len)
{
	lat_micro_t	lat;
	lon_micro_t	lon;
	s_int32 elev;
	u_int32 pitch, bank, heading;

	lat.dec_part 	= get2();
	lat.long_part 	= get4();
	lon.dec_part	= get2();
	lon.long_part	= get4();
	get2();	/* unknown */

	elev		= get4();
	pitch		= get4();
	bank		= get4();
	heading		= get4();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		fprintf(outfp, "%s ", latstr(lat_micro2lat(&lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_micro2lon(&lon), NULL));
		fprintf(outfp, "%d ", elev);
		fprintf(outfp, "%f ", deg322deg(pitch));
		fprintf(outfp, "%f ", deg322deg(bank));
		fprintf(outfp, "%f ", deg322deg(heading));
		fprintf(outfp, ")\n");
	}
}

/*
-- 41h ----- TaxiTo
*/
void anaDynTaxiTo(char *name, u_int8 c, int len)
{
	lat_micro_t	lat;
	lon_micro_t	lon;
	s_int32 elev;
	u_int32 heading;

	lat.dec_part 	= get2();
	lat.long_part 	= get4();
	lon.dec_part	= get2();
	lon.long_part	= get4();

	get2();	/* unknown : padding? */

	elev		= get4();
	heading		= get4();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		fprintf(outfp, "%s ", latstr(lat_micro2lat(&lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_micro2lon(&lon), NULL));
		fprintf(outfp, "%d ", elev);
		fprintf(outfp, "%f ", deg322deg(heading));
		fprintf(outfp, ")\n");
	}
}

/*
-- 43h ----- MonitorRwy
*/
void anaMonitorRwy(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;

	lat_long_t	lat;
	lon_dec_t	lon;
	s_int32 elev;
	u_int16 heading;
	u_int32 rlen, wid;

	basep = GetPointer() - 1;
	offset = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}

	lat = get4() * 65536;
	lon = get4() * 65536;

	elev = get4();
	heading = get2();
	rlen = get4();
	wid = get4();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %s ", latstr(lat_long2lat(lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_dec2lon(lon), NULL));
		fprintf(outfp, "%d ", elev);
		fprintf(outfp, "%f ", deg162deg(heading));
		fprintf(outfp, "%d ", rlen);
		fprintf(outfp, "%d ", wid);
		fprintf(outfp, ")\n");
	}
}
