/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec16ana.c,v 1.5 2000/01/04 06:41:54 tmurakam Exp $
 */
/*
 * Section 16 (IMO Marker, Landme) analyze
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "conv.h"
#include "label.h"
#include "sec16ana.h"

/* prototypes */
void AnalyzeIMOLandme(u_int32 pointer);
void AnalyzeIMO(char *);
void AnalyzeLandme(void);
void AnalyzeTimeZone(void);
static u_int32 anaArea16(u_int8 c);
static void anaSec16Main(u_int32);
static void anaSec16Main2(u_int32);

extern void
AnalyzeSec16(u_int32 sec_pointer)
{
	AnalyzeLatRangeTable(16, sec_pointer, anaSec16Main);
}
	
static void anaSec16Main(u_int32 pointer)
{
	ClearLabel();
	SET_PASS1;
	anaSec16Main2(pointer);
	SET_PASS2;
	anaSec16Main2(pointer);
	ClearLabel();
}

static void anaSec16Main2(u_int32 basepoint)
{
	u_int8	c;
	int i;
	int in_area = 0;
	int endaflag = 0;
	u_int32 p;
	u_int32 enda = 0;

	MovePointer(basepoint);

	while(1) {
		c = get1();

		if (IS_PASS2 && endaflag) {
			fprintf(outfp, "End16\n\n");
			endaflag = 0;
		}

		if (!in_area) {
			switch(c) {
			    case 0x0:
				return;	/* end */
				
			    case 0x04:
				AnalyzeIMO("IM");
				break;

			    case 0x05:
				AnalyzeIMO("MM");
				break;

			    case 0x06:
				AnalyzeIMO("OM");
				break;

			    case 0x07:
				AnalyzeLandme();
				break;

			    case 0x64:
				AnalyzeTimeZone();
				break;

			    case 0x65:
				enda = anaArea16(c);
				in_area = 1;
				break;

			    default:
				/* unknwon record */
				if (IS_PASS2) {
					fprintf(outfp, ";;; unknown record: 0x%x\n", c);
				}
				break;
			}
		}
		else {
			i = 0;

			/* for debug*/
			/*if (IS_PASS2) {
			  fprintf(outfp, ";#");
			  PrintLabel(GetPointer()-1);
			  fprintf(outfp, "\n");
			}*/

			while(sec16tab[i].record_id) {
				if (sec16tab[i].record_id == c) {
					(*sec16tab[i].func)(sec16tab[i].name, c, sec16tab[i].len);
					break;
				}
				i++;
			}
			if (sec16tab[i].record_id == 0) {
				if (IS_PASS2) {
					fprintf(outfp, ";;; unknown record id: 0x%x\n", c);
				}
				/*exit(1);*/
			}

			/* check EndA */
			p = GetPointer();
			if (p >= enda) {
				endaflag = 1;	/* To keep order of EndA and label... */

				if (IS_PASS2) {

					if (p > enda) {
						fprintf(outfp, ";;;;; EndA Overrun. (now:0x%x, enda:0x%x\n", p, enda);
					}
				}
				in_area = 0;
			}
		}
	}
}

/*
-- 04h/05/06h ------ IMO Marker (11 bytes) (Source: JOHN TRINDLE)

1     1   record length
2     3   Latitude  in Iu
5     3   Longitude in Iu
8     2   Altitude in m ??
10    1   ???
*/
void AnalyzeIMO(char *name)
{
	u_int8	len;
	lat_int_t	lat;
	lon_int_t	lon;
	u_int16		alt;

	len = get1();

	lat = get3();
	lon = get3();

	alt = get2();
	get1();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		fprintf(outfp, "%s ", latstr(lat_int2lat(lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_int2lon(lon), NULL));
		fprintf(outfp, "%d )\n", alt);
	}
}

/*
-- 07h ------------- Land-me (24 bytes + airport name)

1     1   record length
2     3   Touchdown point latitude  in Iu
5     3   Touchdown point longitude in Iu
8     3   Touchdown altitude in m
11    3   Another latitude  in Iu (presumably at the other end of the rwy)
15    3   Another longitude in Iu
18    3   Another altitude in m
21    1   ?
22    x   Variable length 0-terminated ASCII string with the airport name
22+x  1   runway 1 ID (bit 6 and 7 may be set: ??)
23+x  1   runway 2 ID (bit 6 and 7 may be set: ??)
*/
void AnalyzeLandme(void)
{
	u_int8	len;
	lat_int_t	lat1, lat2;
	lon_int_t	lon1, lon2;
	u_int32		alt1, alt2;
	char name[100];
	u_int8	id1, id2;
	u_int8	c;
	int	i;

	len = get1();

	lat1 = get3();
	lon1 = get3();
	alt1 = get3();

	lat2 = get3();
	lon2 = get3();
	alt2 = get3();

	get1();
	get1();

	i = 0;
	while (1) {
		c = get1();
		if (c == 0x0) {
			name[i] = 0;
			break;
		}
		name[i] = c;
		i++;
	}
	NormalString(name);

	id1 = get1();
	id2 = get1();

	if (IS_PASS2) {
		fprintf(outfp, "LandMe( ");
		fprintf(outfp, "%s ", latstr(lat_int2lat(lat1), NULL));
		fprintf(outfp, "%s ", lonstr(lon_int2lon(lon1), NULL));
		fprintf(outfp, "%.2f %d\n", alt1 / 256.0, (int)id1);
		fprintf(outfp, "\t%s ", latstr(lat_int2lat(lat2), NULL));
		fprintf(outfp, "%s ", lonstr(lon_int2lon(lon2), NULL));
		fprintf(outfp, "%.2f %d\n", alt2 / 256.0, (int)id2);
		fprintf(outfp, "\t%s )\n", name);
	}
}

/*-- 64h -------------  TimeZone

2     1   record length
3     3   LatMax
6     3   LatMin
9     3   LonMax
12    3   LonMin
15    2   minutes
17    1   ds1
18    1   ds1
*/
void
AnalyzeTimeZone(void)
{
	lat_int_t latmax, latmin;
	lon_int_t lonmax, lonmin;
	u_int16 minutes;
	u_int8 ds1, ds2;

	get1();		/* record len */

	latmax = get3();
	latmin = get3();
	lonmax = get3();
	lonmin = get3();

	minutes = get2();
	ds1 = get1();
	ds2 = get1();

	if (IS_PASS2) {
		fprintf(outfp, "\tTimeZone( ");
		fprintf(outfp, "%s ", latstr(lat_int2lat(latmax), NULL));
		fprintf(outfp, "%s ", latstr(lat_int2lat(latmin), NULL));
		fprintf(outfp, "%s ", lonstr(lon_int2lon(lonmax), NULL));
		fprintf(outfp, "%s ", lonstr(lon_int2lon(lonmin), NULL));
		fprintf(outfp, "%d %d %d )\n", minutes, (int)ds1, (int)ds2);
	}
}

/*
 * Analyze Area16
 */
static u_int32
anaArea16(u_int8 c)
{
	u_int32 basep;

	lat_int_t	lat1, lat2;
	lon_int_t	lon1, lon2;
	u_int16		offset;
	char lats1[LLBLEN], lons1[LLBLEN];
	char lats2[LLBLEN], lons2[LLBLEN];

	basep = GetPointer() - 1;
	
	offset = get1();
	lat1 = get3();
	lat2 = get3();
	lon1 = get3();
	lon2 = get3();


	/* display */
	if (IS_PASS2) {
		latstr(lat_int2lat(lat1), lats1);
		latstr(lat_int2lat(lat2), lats2);
		lonstr(lon_int2lon(lon1), lons1);
		lonstr(lon_int2lon(lon2), lons2);
		fprintf(outfp, "Area16( %s %s %s %s )\n", lats1, lats2, lons1, lons2 );
	}

	return basep + offset;
}

/*-- 22h -------------  Return */
void anaReturn16(char *name, u_int8 c, int len)
{
	get1();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s\n", name);
	}
}

/*-- 24h -------------  IfVarRange (10 bytes)

2     2   byte offset
4     2   LocSeg variable address
6     2   min value
8     2   max value
*/

void
anaIfVarRange16(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 var;
	u_int16 min, max;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	var = get2();
	min = get2();
	max = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %X %d %d )\n", var, min, max);
	}
}

/*-- 33h -------------  RotatedCall (10 bytes)

2     2   byte offset
4     2   rotation around x axis (East)
6     2   rotation around y axis (alt)
8     2   rotation around z axis (North)
*/
void 
anaRotatedCall16(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	double xdeg, ydeg, zdeg;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	xdeg = deg162deg(get2());
	ydeg = deg162deg(get2());
	zdeg = deg162deg(get2());

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %.2f %.2f %.2f )\n", xdeg, ydeg, zdeg);
	}
}

/*-- 2F/77h -------------  RefPoint (32 bytes)

1   1    dummy
2   2    offset
4   2    V1=
6   2    V2=
8   4    unknown
12  4    scale factor
16  6    lat
22  6    lon
28  4    E= (elevation)
*/
void
anaRefPoint16(char *name, u_int8 c, int len)
{
	u_int32 basep;

	char type;
	s_int16 offset;
	u_int32	scale;
	lat_long_t	lat;
	lon_dec_t	lon;
	u_int32		alt;
	u_int16 v1, v2;

	basep = GetPointer() - 1;

	switch (c) {
	case 0x2f:
		type = '2';
		break;

	case 0x77:
		type = '7';
		break;
	}

	get1();
	offset = get2();	/* jump offset */
	v1 = get2();
	v2 = get2();
	get2(); /* unknown */
	get2(); /* unknown */
	scale = get4();
	lat = get4();
	get2();
	lon = get4();
	get2();
	alt = get4();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\tRefPoint( %c ", type);
		PrintLabel(basep + offset);
		fprintf(outfp, " %d ", scale);
		fprintf(outfp, "%s ", latstr(lat_long2lat(lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_dec2lon(lon), NULL));
		if (type == '2') {
			fprintf(outfp, "E= %d ", alt);
		}
		fprintf(outfp, "v1= %d v2= %d )\n", v1, v2);
	}
}

/*-- 39h -------------  IfVarAnd (8 bytes)

2     2   byte offset
4     2   LocSeg variable address
6     2   value
*/
void
anaIfVarAnd16(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 var, val;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	var = get2();
	val = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %X %X )\n", var, val);
	}
}

/*-- 71h -------------  SetElevation (4 bytes)

2     2   Elevation
*/
void anaSetElevation(char *name, u_int8 c, int len)
{
	s_int16 value;

	get1();
	value = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d )\n", name, value);
	}
}

/*-- 70h -------------  SenseBorder (6 bytes + 4*NumberOoutfpoints)

2     2   offset
2     2   number of points

then for each point:

x     2   delta East coordinate
x+2   2   delta North coordinate
*/
void
anaSenseBorder16(char *name, u_int8 c, int len)
{
	u_int32	basep;

	int numvert, i;
	s_int16 offset;
	s_int16 x, y;

	basep = GetPointer() - 1;

	get1();/* align */
	offset = get2();
	numvert = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
		ProcPointer(4 * numvert);
	} else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, "\n");

		for (i=0; i<numvert; i++) {
			x = get2();
			y = get2();
			fprintf(outfp, "\t\t%d %d\n", x, y);
		}
		fprintf(outfp, "\t)\n");
	}
}

