/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec20ana.c,v 1.4 2000/01/04 06:42:13 tmurakam Exp $
 */
/*
 * Section 20 (Airport) analyze (experimental)
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "conv.h"
#include "sec20ana.h"

#if 0	/* ### NOT YET */

#define	SEC20_HEADER_POS	0x80
#define	SEC20_INSTDATA_OFF	0x1c
#define	SEC20_INSTNAME_OFF	0x24

/* prototypes */

extern void
AnalyzeSec20(u_int32 sec_pointer)
{
	char fname[100];
	long instdata_pos, instname_pos;	/* ### */

 	SET_PASS2;	/* no PASS 1 */

	/* open output file */
	sprintf(fname, "%s20", SECFILENAME_PRE);
	OpenOutFile(fname, TRUE);

	/* analyze header */
	MovePointer(SEC20_HEADER_POS + SEC20_INSTDATA_OFF);
	instdata_pos = get4();

	MovePointer(SEC20_HEADER_POS + SEC20_INSTNAME_OFF);
	instname_pos = get4();

	/* analyze installation data */
	MovePointer(instdata_pos);
	AnalyzeInstData();
	
	/* analyze installation name */
	MovePointer(instname_pos);
	AnalyzeInstName();

	CloseOutFile();
}

void AnalyzeInstData(void)
{
	u_int32	header_id;
	u_int32 len;
	u_int16 type;
	char	*typename;

	while (1) {
		header_id = get4();
		if (header_id == 0) {
			/* End of record */
			break;
		}

		if (header_id != 1) {
			/* Illegal record ! */
			/* ### */
		}

		/* header-A */
		len = get4();	/* length */
		type = get2();	/* type */
		get4();		/* dummy */

		switch (type) {
		    case 1:
			typename = "AIRPORT";
			break;
		
		    case 2:
			typename = "VOR";
			break;

		    case 3:
			typename = "NDB";
			break;

		    default:
			typename = "???";
			break;
		}

		fprintf(outfp, "; InstData [ %s ]\n", typename);

		AnalyzeInstDataBody();
	}
}

void AnalyzeInstDataBody(void)
{
	u_int16 type1, type2;
	u_int32 len;

	while (1) {
		type1 = get2();
		type2 = get2();

		if (type1 == 0) {
			/* end of records */
			return;
		}

		len = get4();

		switch (type1) {
		    case 1:
			switch (type2) {
			    case 1:
				/* airport position */
			    case 2:
				/* COM frequency */
			    default:
			}
			break;

		    case 2:
			if (type2 == 1) {
				/* Runway center */
			}
			break;
			
		    case 4:
			if (type2 == 1) {
				/* NAVAID */
			}
			break;

		    case 5:
			if (type2 == 1) {
				/* Runway end */
			}
			break;

		    case 6:
			if (type2 == 1) {
				/* airport code */
			}
			break;

		    default:
			/* ### */
		}
	}
}
			


void AnalyzeAirport(void)
{
	u_int8	c;

	while (1) {
		c = get1();
		if (c == 0) break;

		if (c != 0x03) {
			fprintf(stderr, "unknown record\n");
			exit(1);
		}

		AnalyzeAirportMain();
	}
}

/*
-- 03h ------------- Airport (27 bytes + airport name)

1     2   record length
3     4   Latitude  (in Du)
7     4   Longitude (in Du)
11    4   Altitude in 1/256 meters
15    2   Runway heading in Deg16
17    2   COM1 frequency (BCD coded: 30h 21h = 121.30)
19    2   NAV1 frequency (as above)
21    2   OBI1 course
23    2   NAV2 frequency (as above)
25    2   OBI2 course
27    -   Variable length 0-terminated ASCII string with the airport and
          runway name
*/
void AnalyzeAirportMain(void)
{
	u_int16		len;
	lat_dec_t	lat;
	lon_dec_t	lon;
	u_int32		alt;
	u_int16		heading;
	u_int16		com1, nav1, obi1, obi2, nav2;
	char		name[100];
	int i;

	len = get2();
	lat = get4();
	lon = get4();
	alt = get4();
	heading =get2();
	com1 = get2();
	nav1 = get2();
	obi1 = get2();
	nav2 = get2();
	obi2 = get2();
	for (i=0; i<100; i++) {
		name[i] = get1();
		if (name[i] == 0) break;
		if (name[i] == '(') name[i] = '{';
		if (name[i] == ')') name[i] = '}';
		if (name[i] == ' ') name[i] = '_';
	}
	name[99] = 0;	/* ad hoc... */

	fprintf(outfp, "MenuEntry(\n");
	fprintf(outfp, "\t%s\n", latstr(lat_dec2lat(lat), NULL));
	fprintf(outfp, "\t%s\n", lonstr(lon_dec2lon(lon), NULL));
	fprintf(outfp, "\t%.2f\n", (alt / 256.0) + 0.005);
	fprintf(outfp, "\t%.2f\n", deg162deg(heading) + 0.005);
	fprintf(outfp, "\t1%d%d.%d%d\n", com1 >> 12, (com1 >> 8) & 0xf, (com1 >> 4) & 0xf, com1 & 0xf);
	fprintf(outfp, "\t1%d%d.%d%d\n", nav1 >> 12, (nav1 >> 8) & 0xf, (nav1 >> 4) & 0xf, nav1 & 0xf);
	fprintf(outfp, "\t%d\n", obi1);
	fprintf(outfp, "\t1%d%d.%d%d\n", nav2 >> 12, (nav2 >> 8) & 0xf, (nav2 >> 4) & 0xf, nav2 & 0xf);
	fprintf(outfp, "\t%d\n", obi2);
	fprintf(outfp, "\t%s )\n", name);
}

#endif
