/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec9ana.c,v 1.9 2000/01/04 06:42:19 tmurakam Exp $
 */
/*
 * Section 9 : Analyze Visual scenery
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "label.h"
#include "conv.h"
#include "sec9ana.h"

/* global variables */
extern SecRecordTab sec9tab[];

/* prototypes */
static void anaSec9Main(u_int32 poitner);
static u_int32 anaArea(u_int8);
static void anaSynMain(u_int32);

extern void 
AnalyzeSec9(u_int32 sec_pointer)
{
	AnalyzeLatRangeTable(9, sec_pointer, anaSec9Main);
}

/*#define	SKIP_PASS1*/	/* for debug */

/* start analyze */
void anaSec9Main(u_int32 pointer)
{
		ClearLabel();
#ifndef	SKIP_PASS1
		SET_PASS1;
		anaSynMain(pointer);
#endif
		SET_PASS2;
		anaSynMain(pointer);
		ClearLabel();
}

static void
anaSynMain(u_int32 basepoint)
{
	u_int8 c;
	int i;
	int in_area = 0;
	u_int32 enda = 0;
	u_int32 p;
	int endaflag = 0;

	MovePointer(basepoint);

	while (1) {

		c = get1();
		if (IS_PASS2 && endaflag) {
			fprintf(outfp, "EndA\n\n");
			endaflag = 0;
		}

		if (!in_area) {
			switch(c) {
			case 0x05:
				enda = anaArea(c);
				in_area = 1;
				break;
			case 0x08:
				enda = anaArea(c);
				in_area = 1;
				break;
			case 0x0b:
				enda = anaArea(c);
				in_area = 1;
				break;
			case 0x0:	/* ExitSDL */
			case 0x16:	/* ExitSDL16 */
			case 0x17:	/* ExitSDL17 */
				if (IS_PASS2) {
					fprintf(outfp, ";;;;; End of LatRange\n");
				}
				return;
			default:
				if (IS_PASS2) {
					fprintf(outfp, ";;; unknown record: 0x%x\n", c);
				}
				break;
			}
			
		}
		else {
			i = 0;

			/* for debug*/
			/*if (IS_PASS2) {
				fprintf(outfp, ";#");
				PrintLabel(GetPointer()-1);
				fprintf(outfp, "\n");
			}*/

			while(sec9tab[i].record_id) {
				if (sec9tab[i].record_id == c) {
					(*sec9tab[i].func)(sec9tab[i].name, c, sec9tab[i].len);
					break;
				}
				i++;
			}
			if (sec9tab[i].record_id == 0) {
				if (IS_PASS2) {
					fprintf(outfp, ";;; unknown record id: 0x%x\n", c);
				}
				/*exit(1);*/
			}

			/* check EndA */
			p = GetPointer();
			if (p >= enda) {
				endaflag = 1;	/* To keep order of EndA and label... */

				if (IS_PASS2) {

					if (p > enda) {
						fprintf(outfp, ";;;;; EndA Overrun. (now:0x%x, enda:0x%x\n", p, enda);
					}
				}
				in_area = 0;
			}
		}
	}
}

/*
 * Analyze Area??
 */
static u_int32
anaArea(u_int8 c)
{
	u_int32 basep;

	char		type;
	lat_long_t	lat;
	lon_dec_t	lon;
	u_int8		range;
	u_int16		offset;
	char lats[LLBLEN], lons[LLBLEN];

	basep = GetPointer() - 1;
	
	lat = get4();
	lon = get4();
	range = get1();
	offset = get2();

	/* display */
	if (IS_PASS2) {
		switch (c) {
		case 0x05:
			type = '5';
			break;
		case 0x08:
			type = '8';
			break;
		case 0x0b:
			type = 'B';
			break;
		}


		latstr(lat_long2lat(lat), lats);
		lonstr(lon_dec2lon(lon), lons);
		fprintf(outfp, "Area( %c %s %s %d )\n", type, lats, lons, range);
	}

	return basep + offset;
}

/*
 * Analyze unknown record which length is known.
 */
void
anaUnknown(char *name, u_int8 c, int len)
{
	int i;
	if (IS_PASS2) {
		fprintf(outfp, ";;; %s(0x%02x) - unknown record\n", name, c);
	}
	if (len < 1) return; /* ad hoc */

	if (IS_PASS2) {
		fprintf(outfp, "\tDbx( %X ", c);
	}
	for (i=0; i<len - 1; i++) {
		c = get1();
		if (IS_PASS2) {
			fprintf(outfp, "%X ", c);
		}
	}
	if (IS_PASS2) {
		fprintf(outfp, ")\n");
	}
}

/*
 * Dump record which scasm does not support.
 */
void
anaDump(char *name, u_int8 c, int len)
{
	int i;
	if (IS_PASS2) {
		fprintf(outfp, ";;; %s(0x%02x) - SCASM does not support this record\n", name, c);
	}
	if (len < 1) return; /* ad hoc */

	if (IS_PASS2) {
		fprintf(outfp, "\tDbx( %X ", c);
	}
	for (i=0; i<len - 1; i++) {
		c = get1();
		if (IS_PASS2) {
			fprintf(outfp, "%X ", c);
		}
	}
	if (IS_PASS2) {
		fprintf(outfp, ")\n");
	}
}

/*
 * Skip record
 */
void
anaSkip(char *name, u_int8 c, int len)
{
	if (IS_PASS2) {
		fprintf(outfp, ";;; %s - skipped\n", name);
	}
	if (len < 1) return; /* ad hoc */
	ProcPointer(len - 1);
}

/* for instruction which have no arguments */
/*-- 05h -------------  StartSurface (2 bytes)*/
void anaNoArg(char *name, u_int8 c, int len)
{
	get1();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s\n", name);
	}
}

/* for instruction which have 1 word hex arguments */
/*-- 14h -------------  SurfaceCol14 (4 bytes)

2     2   colour
*/
void anaArgWx(char *name, u_int8 c, int len)
{
	u_int16 value;

	get1();
	value = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %X )\n", name, value);
	}
}

/* for instruction which have 1 word decimal arguments */
/*-- 1Eh -------------  Set1ED6 (4 bytes)

2     2   value to be copied into FSDS var 1ED6 (?)
*/
void anaArgWd(char *name, u_int8 c, int len)
{
	s_int16 value;

	get1();
	value = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d )\n", name, value);
	}
}

/* for instruction which have long word hex arguments */
void anaArgLx(char *name, u_int8 c, int len)
{
	u_int32 value;

	get1();
	value = get4();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %X )\n", name, value);
	}
}



/*
-- 01h -------------  Forever (2 bytes)

This record never skips its two bytes forcing the SDL5 interpreter to enter a
never ending loop.
*/
void anaForever(char *name, u_int8 c, int len)
{
	get1();/*dummy*/
	if (IS_PASS2) {
		fprintf(outfp, ";;; %s (Infinite loop)\n", name);
	}
}

/*
-- 02h -------------  Nop (2 bytes)

This record does nothing.  It can be used to stuff unused scenery portion or
to overwrite obsolete details.  It share the same code with records 1Bh, 1Ch,
2Bh and 43h.
*/
void anaNop(char *name, u_int8 c, int len)
{
	get1();/*dummy*/

	if (IS_PASS2) {
		fprintf(outfp, ";;; %s \n", name);
	}
}

/*
-- 03h -------------  Switch (8 bytes + 4*NumOfCases)

2     2   Number of cases
4     2   LocSeg variable address
6     2   byte offset for the default case
8     2   byte offset for var value == 0
10    2   byte offset for var value == 1
...
*/
void anaSwitch(char *name, u_int8 c, int len)
{
	int numcase, i;
	u_int16 varaddr;
	s_int16 defoffset, *offset;/*###*/
	u_int32 basep;

	basep = GetPointer() - 1;

	get1();/*dummy*/
	numcase = get2();
	varaddr = get2();
	defoffset = get2();
	if (IS_PASS1) {
		PutLabel(basep + defoffset);
	}

	offset = (s_int16 *)malloc(numcase * sizeof(s_int16));
	if (!offset) {
		fprintf(stderr, "no enough memory");
		exit(1);
	}

	for (i=0; i<numcase; i++) {
		offset[i] = get2();
		if (IS_PASS1) {
			PutLabel(basep + offset[i]);
		}
	}

	/* SCASM does not support Switch instruction, so this will be simulated
	   by multiple IfVarRange instructions. */
	if (IS_PASS2) {
		fprintf(outfp, "\t;;; Switch instruction is not implemented in scasm.\n");
		fprintf(outfp, "\t;;; So bgl2scm simulates this by IfVarRange.\n");
		for (i=0; i<numcase; i++) {
			fprintf(outfp, "\tIfVarRange( ");
			PrintLabel(basep + offset[i]);
			fprintf(outfp, " %X %d %d )\n", varaddr, i, i);
		}
		fprintf(outfp, "\tJump( ");
		PrintLabel(basep + defoffset);
		fprintf(outfp, " )\n\n");
	}

	free(offset);
}

/*-- 04h -------------  NopBX (2 bytes)*/
void anaNopBX(char *name, u_int8 c, int len)
{
	u_int8 c2;
	c2 = get1();

	if (IS_PASS2) {
		fprintf(outfp, ";;; NopBX\n");
		fprintf(outfp, "\tDbx( %X %X )\n", c, c2);
	}
}

/*-- 06h ------------- MoveTo (8 bytes)

2     2   delta East   coordinate
4     2   delta Altit. coordinate
6     2   delta North  coordinate
*/
void anaXTo(char *name, u_int8 c, int len)
{
	s_int16 x, y, z;
	get1();
	x = get2();
	z = get2();
	y = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, x, z, y);
	}
}

/*-- 0Ah ------------- MoveTo0A (10 bytes)

2     2   delta East   coordinate
4     2   delta Altit. coordinate
6     2   delta North  coordinate
8     2   ??
*/
void anaXTo10(char *name, u_int8 c, int len)
{
	s_int16 x, y, z;
	get1();
	x = get2();
	z = get2();
	y = get2();
	get2(); /* dummy */

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, x, z, y);
	}
}

/*-- 0Dh -------------  Jump (4 bytes)

2     2   Jump byte offset
*/
void 
anaJumpCall(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;

	basep = GetPointer() - 1;

	get1();
	offset = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " )\n");
	}
}

/*-- 0Eh -------------  DefPoint (10 bytes)

2     2   point no.
4     2   delta East coordinate
6     2   delta Altit. coordinate
8     2   delta North coordinate
*/
void
anaDefPoint(char *name, u_int8 c, int len)
{
	int number;
	s_int16 x, y, z;

	get1();
	number = get2();
	x = get2();
	z = get2();
	y = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d %d )\n", name, number, x, z, y);
	}
}

/*-- 0Fh -------------  MoveToPoint (4 bytes)

2     2   point no.
*/
void
anaXToPt(char *name, u_int8 c, int len)
{
	int number;

	get1();
	number = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d )\n", name, number);
	}
}

/*-- 11h -------------  DefPPoint (12 bytes)

2     2   point no.
4     2   value for FSDS var 14D8
6     2   delta East   coordinate
8     2   delta Altit. coordinate
10    2   delta North  coordinate
*/
void
anaDefPPoint(char *name, u_int8 c, int len)
{
	int number;
	u_int16 value;
	s_int16 x, y, z;

	get1();
	number = get2();
	value = get2();
	x = get2();
	z = get2();
	y = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t;;; DefPPoint instruction is not supported by SCASM.\n");
		fprintf(outfp, "\t;;; DefPPoint %d %d %d %d %d\n", number, value, x, z, y );
		fprintf(outfp, "\tDbx( %X 0 )", c);
		fprintf(outfp, "\tDbw( %d %d %d %d %d )", number, value, x, z, y);
	}
}

/*-- 12h -------------  MoveToPPoint (4 bytes)

2     2   point no.
*/
void
anaXToPPt(char *name, u_int8 c, int len)
{
	int number;

	get1();
	number = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t;;; %s instruction isnot supported by SCASM.\n", name);
		fprintf(outfp, "\t;;; %s %d\n", name, number);
		fprintf(outfp, "\tDbx( %X 0 )", c);
		fprintf(outfp, "\tDbw( %d )\n", number);
	}
}

/*-- 15h -------------  TexRelief (18 bytes + (ECount+1)*(NCount+1) )

2     2   East  point count
4     2   North point count
6     2   East  increment (delta units)
8     2   North increment (delta units)
10    2   East  initial   (delta units)
12    2   North initial   (delta units)
14    2   min Alt ?
16    2   max Alt ?

then, for each of the (ECount+1)(NCount) points:

x     1   Texture X
x+1   1   Texture Y
x+2   2   Alt (delta units)
*/
void
anaTexRelief(char *name, u_int8 c, int len)
{
	u_int16 x, y;
	s_int16 wx, wy, dx, dy;
	u_int8 px, py;
	s_int16 minalt, maxalt, alt;
	unsigned int eindx, nindx;

	get1();
	x = get2();
	y = get2();
	wx = get2();
	wy = get2();
	dx = get2();
	dy = get2();
	minalt = get2();
	maxalt = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d %d %d %d\n", 
			name, x, y, wx, wy, dx, dy);
	}

	for (eindx = 0; eindx <= x; eindx++) {
		for (nindx = 0; nindx <= y; nindx++) {
			px = get1();
			py = get1();
			alt = get2();
			if (IS_PASS2) {
				fprintf(outfp, "\t\t%d %d %d\n",  px, py, alt);
			}
		}
	}
	if (IS_PASS2) {
		fprintf(outfp, "\t)\n");
	}
}

/*-- 18h ------------- Texture (24 bytes)

2     2   ???
4     2   delta East
6     2   scale?
8     2   delta North
10    8   texture file name (padded with spaces)
18    4   texture file .ext (with the period and padded with spaces)
22    2   ???
*/
void
anaBitmap(char *name, u_int8 c, int len)
{
	s_int16 x, dx, scale, dy;
	char fname[8 + 1], ext[4 + 1], filename[20];

	get1();
	x = get2();
	dx = get2();
	scale = get2();
	dy = get2();
	getlong(fname, 8);
	getlong(ext, 4);
	get2();

	RemoveTailSpace(fname, 8);
	RemoveTailSpace(ext, 4);

	if (ext[0] == '\0') {
		sprintf(filename, "%s", fname);
	} else {
		sprintf(filename, "%s%s", fname, ext);
	}

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %s %d %d %d %d )\n", name, filename, x, dx, scale, dy);
	}
}

/*-- 19h ------------- Palette (16 bytes)

2     8   palette file name (padded with spaces)
10    4   palette file .ext (with the period and padded with spaces)
14    2   ???

This record loads and activates a palette file in the TEXTURE directory.  The
palette file has to contain 256 3-byte colour values, but only the last 64
are actually activated (see above, under .PAL file discussion).
*/
void
anaPalette(char *name, u_int8 c, int len)
{
	char fname[8 + 1], ext[4 + 1];
	char filename[20];

	get1();
	getlong(fname, 8);
	getlong(ext, 4);
	get2();

	RemoveTailSpace(fname, 8);
	RemoveTailSpace(ext, 4);

	if (ext[0] == '\0') {
		sprintf(filename, "%s", fname);
	} else {
		sprintf(filename, "%s%s", fname, ext);
	}

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %s )\n", name, filename);
	}
}

/*-- 1Ah -------------  Defpoints (6 bytes + 6*NumOoutfpoints)

2     2   first point no.
4     2   number of points

then, for each point:

x     2   delta East coordinate
x+2   2   delta Alt. coordinate
x+4   2   delta North coordinate
 */
void
anaPoints(char *name, u_int8 c, int len)
{
	int i, numpt;
	u_int16 firstpt;
	s_int16 x, y, z;

	get1();	/* align */
	firstpt = get2();
	numpt  = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d\n", name, firstpt);

		for (i = 0; i<numpt; i++) {
			x = get2();
			z = get2();
			y = get2();

			fprintf(outfp, "\t\t%d %d %d\n", x, z, y);
		}
		fprintf(outfp, "\t)\n");
	} else {
		ProcPointer(6 * numpt);
	}
}

/*
-- 1Bh -------------  MonitorTr (22 bytes)

2     2   offset
4     2   dx
6     2   dz
8     2   dy
10    2   wx
12    2   wz
14    2   wy
16    2   xdeg   
18    2   ydeg
20    2   zdeg
*/
void
anaMonitorTr(char *name, u_int8 c, int len)
{
	s_int16 offset;
	s_int16 dx, dz, dy, wx, wz, wy, xdeg, zdeg, ydeg;
	u_int32 basep;

	basep = GetPointer() - 1;

	get1(); /* dummy */
	offset = get2();

	dx = get2();
	dz = get2();
	dy = get2();
	wx = get2();
	wz = get2();
	wy = get2();
	xdeg = get2();
	zdeg = get2();
	ydeg = get2();
	
	if (IS_PASS1) {
		PutLabel(basep + offset);
	}

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, "\n");
		fprintf(outfp, "\t\t%d %d %d\n", dx, dz, dy);
		fprintf(outfp, "\t\t%d %d %d\n", wx, wz, wy);
		fprintf(outfp, "\t\t%d %d %d\n", xdeg, zdeg, ydeg);
	}
}

/*-------------------------------------------------------------------------*/
/*  GoIfOut2 (14 bytes )      Rec. 1Ch Source: Alessandro Antonini         */
/*                                                                         */
/* 2     2   offset                                                        */
/* 2     2   var1                                                          */
/* 3     2   v1val1                                                        */
/* 6     2   v1val2                                                        */
/* 8     2   var2                                                          */
/* 10    2   v2val1                                                        */
/* 12    2   v2val2                                                        */
/* Jump to Label Offset: if two variables are out of range defined with    */
/* v1val1, v1val2 and v2val1, v2val2                                       */
/*                                                                         */
/*-------------------------------------------------------------------------*/
void anaIfVarRange2(char *name, u_int8 c, int len)
{
    u_int32 basep;
    u_int32 offset;
    u_int16 var1;
    s_int16 minval1, maxval1;
    u_int16 var2;
    s_int16 minval2, maxval2;

    basep = GetPointer() - 1;
    get1();
    offset = get2();
    var1 = get2();
    minval1 = get2();
    maxval1 = get2();
    var2 = get2();
    minval2 = get2();
    maxval2 = get2();

    if (IS_PASS1) {
        PutLabel(basep + offset);
    }
    else {
        fprintf(outfp, "\t%s( ", name);
        PrintLabel(basep+offset);
        fprintf(outfp, " %x %d %d  %x %d %d )\n", 
			var1, minval1, maxval1,
			var2, minval2, maxval2);
    }
}


/*
-- 1D -------------- VecPoly (16 bytes + 2*NumOfVertices)

2     2   number of vertices
4     2   point delta East coord.
6     2   point delta Alt. coord.
8     2   point delta North coord.
10    2   vector East comp.
12    2   vector Alt. comp.
14    2   vector North. comp.
*/
void
anaVecPoly(char *name, u_int8 c, int len)
{
	unsigned int	i;
	u_int8 dummy;
	u_int16 numvert, n;
	s_int16 dx, dy, dz, vx, vy, vz;

	dummy = get1();
	numvert = get2();
	dx = get2();
	dz = get2();
	dy = get2();
	vx = get2();
	vz = get2();
	vy = get2();
	
	if (IS_PASS2) {
		fprintf(outfp, "\t%s( m  %d %d %d  %d %d %d\n", name, dx, dz, dy, vx, vz, vy);

		for (i = 0; i<numvert; i++) {
			n = get2();
			fprintf(outfp, "\t\t%d ", n);
		}
		fprintf(outfp, ")\n");
	} else {
		ProcPointer(2 * numvert);
	}
}


/*
-- 20h -------------  TexturePolygon (14 bytes + 6*NumOfVertices)

2     2   number of vertices
4     2   vector East component
6     2   vector Alt. component
8     2   vector North component
10    4   distance from the ref. point (in delta units * vector modulus)

then, for each point:

x     2   point number
x+2   2   texture x coordinate (0-255)
x+4   2   texture y coordinate (0-255)
*/
void
anaTexPoly(char *name, u_int8 c, int len)
{
	int numvert, i;
	s_int16 vx, vz, vy;
	u_int16 tx, ty;
	u_int16 number;
	double dist;

	get1();/*align*/
	numvert = get2();
	vx = get2();
	vz = get2();
	vy = get2();
	dist = (double)((s_int32)get4()) / 32768.0;

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( m %d %d %d %.2f", name, 
			vx, vz, vy, dist);

		for (i=0; i<numvert; i++) {
			number = get2();
			tx = get2();
			ty = get2();

			fprintf(outfp, "\t\t%d %d %d", number, tx, ty);
		}

		fprintf(outfp, "\t)\n");
	} else {
		ProcPointer(6 * numvert);
	}
}


/*--------------------------------------------------------------------------
GoIfOut3 (Label, Var1, minVar1, maxVar1, Var2, minVar2, maxVar2, Var3, 
minVar3, maxVar3 ) 
    record 0x21
--------------------------------------------------------------------------*/
void anaIfVarRange3(char *name, u_int8 c, int len)
{
    u_int32 basep;
	u_int8 d1;
    s_int16 offset;
    u_int16 var1;
    u_int16 min1, max1;
    u_int16 var2;
    u_int16 min2, max2;
    u_int16 var3;
    u_int16 min3, max3;

    basep = GetPointer() - 1;

    d1 = get1();
    offset = get2();
    var1 = get2();
    min1 = get2();
    max1 = get2();
    var2 = get2();
    min2 = get2();
    max2 = get2();
    var3 = get2();
    min3 = get2();
    max3 = get2();

    if (IS_PASS1) {
        PutLabel(basep + offset);
    }
    else {
        fprintf(outfp, ";;; %s\n", name);
		fprintf(outfp, "\tDbx( %x %x )\n", c, d1);
		fprintf(outfp, "\tDwd( %d )\t; ", offset);
		PrintLabel(basep + offset);
		fprintf(outfp, "\n");
		fprintf(outfp, "\tDwx( %x )\t; var1\n", var1);
		fprintf(outfp, "\tDwd( %d %d )\t; min1, max1\n", min1, max1);
		fprintf(outfp, "\tDwx( %x )\t; var2\n", var2);
		fprintf(outfp, "\tDwd( %d %d )\t; min2, max2\n", min2, max2);
		fprintf(outfp, "\tDwx( %x )\t; var3\n", var3);
		fprintf(outfp, "\tDwd( %d %d )\t; min3, max3\n", min3, max3);
    }
}

/*-- 24h -------------  JumpOnVar (10 bytes)

2     2   byte offset
4     2   LocSeg variable address
6     2   min value
8     2   max value

This record jumps to the given byte offset if the LocSeg var is outside the
min-max range.*/
void
anaIfVarRange(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 var;
	u_int16 min, max;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	var = get2();
	min = get2();
	max = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %X %d %d )\n", var, min, max);
	}
}


/*-- 25h -------------  JumpOnVect (14 bytes)

2     2   byte offset
4     2   vector East component
6     2   vector Alt. component
8     2   vector North component
10    4   distance from the ref. point (in delta units * vector modulus)

This record is a visibility test: given the plane perpendicular to the ref.
point and at the given distance from the ref. point, if the view point is in
the "lower" side (according to the vector) of the plan, execution jumps to
the given offset, otherwise it continues to the next record.*/

void
anaVectorJump(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	s_int16 x, z, y;
	double dist;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	x = get2();
	z = get2();
	y = get2();
	dist = (double)((s_int32)get4()) / 32767.0;

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " m %d %d %d %f )\n", x, z, y, dist);
	}
}

/*-- 26h -------------  SetLocVar (6 bytes)

2     2   LocSeg variable address
4     2   value

This record sets the given LocSeg variable to the given value.*/

void
anaSetVar(char *name, u_int8 c, int len)
{
	u_int16 var, val;

	get1();
	var = get2();
	val = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %X %d )\n", name, var, val);
	}
}

/*-- 28h -------------  BigDot (10 bytes)   Source: Alessandro Antonini

2     2   Size ( 128, 256 or 512 ??? )
4     2   Delta East
6     2   Delta Alt   (MSL)
8     2   Delta North
*/

void anaBigDot(char *name, u_int8 c, int len)
{
    u_int16 dsize; 
    s_int16 dx, dz, dy; /* -32767 to 32767 */

    get1();
    dsize = get2();
    dx = get2();
    dz = get2();
    dy = get2();

    if (IS_PASS2) {
        fprintf(outfp, "\t%s( %d %d %d %d )\n", 
                             name, dsize, dx, dz, dy);
    }
}

/*
-- 29h -------------  DefVectPoints (6 bytes + 12*NumOoutfpoints)

2     2   first point no.
4     2   number of points

then, for each point:

x     2   delta East coordinate
x+2   2   delta Altit. coordinate
x+4   2   delta North coordinate
x+6   2   vector East component
x+8   2   vector Alt. component
x+10  2   vector North component
*/
void
anaVecPoints(char *name, u_int8 c, int len)
{
	int i, numpt;
	u_int16 firstpt;
	s_int16 x, y, z, vx, vy, vz;

	get1();	/* align */
	firstpt = get2();
	numpt  = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d\n", name, firstpt);

		for (i = 0; i<numpt; i++) {
			x = get2();
			z = get2();
			y = get2();
			vx = get2();
			vz = get2();
			vy = get2();
			fprintf(outfp, "\t\t%d %d %d   %d %d %d\n", x, z, y, vx, vz, vy);
		}
		fprintf(outfp, "\t)\n");
	} else {
		ProcPointer(12 * numpt);
	}
}

/*-- 30h -------------  SurfColIntensity (4 bytes)

2     2   colour intensity (0 = darkest, FFFFh = brightest)
*/
void
anaBrightness(char *name, u_int8 c, int len)
{
	double brightness;

	get1();
	brightness = (double)(u_int16)get2() / 32767.0 * 100.0;	/* % \ */

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d )\n", name, (int)(brightness + 0.5));
	}
}

/*-- 31h -------------  DefPointArray (18 bytes)

2     2   first point no.
4     2   number of points
6     2   P1 delta East
8     2   P1 delta Alt
10    2   P1 delta North
12    2   P2 delta East
14    2   P2 delta Alt
16    2   P2 delta North
*/
void
anaAutoPoints(char *name, u_int8 c, int len)
{
	u_int16 firstpt, numpt;
	s_int16 x1, z1, y1;
	s_int16 x2, z2, y2;

	get1();
	firstpt = get2();
	numpt = get2();
	x1 = get2();
	z1 = get2();
	y1 = get2();
	x2 = get2();
	z2 = get2();
	y2 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d  %d %d %d  %d %d %d )\n",
			name, firstpt, numpt, x1, z1, y1, x2, z2, y2 );
	}
}

/*-- 33h -------------  RotoCall (10 bytes)

2     2   byte offset
4     2   rotation around x axis (East)
6     2   rotation around y axis (alt)
8     2   rotation around z axis (North)
*/
void 
anaRotatedCall(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	double xdeg, ydeg, zdeg;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	xdeg = deg162deg(get2());
	ydeg = deg162deg(get2());
	zdeg = deg162deg(get2());

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %.2f %.2f %.2f )\n", xdeg, ydeg, zdeg);
	}
}

/*--------------------------------------------------------------------------*/
/*   34h -  SetScaleX     - 10 bytes                                        */
/*   2     2   Jump byte offset                                             */
/*   4     2   Scale Factor                                                 */
/*   6     2   v1 (???)                                                     */
/*   8     2   sx (???)                                                     */
/*--------------------------------------------------------------------------*/
/* by Alessandro Antonini */
void 
anaSetScaleX(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 v1, v2, sx;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	v1 = get2();
	v2 = get2();
	sx = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %d %d %d )\n", v1, v2, sx);
	}
}



/*-- 35h ------------- DottedLine (16 bytes)

2     2   delta East  coord. of Point 1
4     2   delta Alt.  coord. of Point 1
6     2   delta North coord. of Point 1
8     2   delta East  coord. of Point 2
10    2   delta Alt.  coord. of Point 2
12    2   delta North coord. of Point 2
14    2   number of dots*/

void 
anaDotLine(char *name, u_int8 c, int len)
{
	s_int16 x1, z1, y1;
	s_int16 x2, z2, y2;
	u_int16 numdot;

	get1();
	x1 = get2();
	z1 = get2();
	y1 = get2();
	x2 = get2();
	z2 = get2();
	y2 = get2();
	numdot = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d  %d %d %d  %d )\n", 
			name, x1, z1, y1, x2, z2, y2, numdot);
	}
}

/*-- 37h ------------- Dot (8 bytes)

2     2   delta East  coord
4     2   delta Alt.  coord
6     2   delta North coord
*/

void 
anaDot(char *name, u_int8 c, int len)
{
	s_int16 x, z, y;

	get1();
	x = get2();
	z = get2();
	y = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, x, z, y );
	}
}

/*-- 39h -------------  TestVar (8 bytes)

2     2   byte offset
4     2   LocSeg variable address
6     2   value
*/
void
anaIfVarAnd(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 var, val;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	var = get2();
	val = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %X %X )\n", var, val);
	}
}



/*-- 41h ------------- ShadowCallVI -*/
void
anaShadowCallVI(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 var;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	var = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %x )\n", var);
	}
}

/*-- 44h -------------  Runway (64 bytes) (source: Adam Szofran)

2     6   Latitude in Mu
8     6   Longitude in Mu
14    6   Altitude in Mu
20    2   Heading in Deg16
22    2   Length in feet
24    2   Width in feet
26    1   Markers: bits  0 ??
                         1 Threshold marker
                         2 Touchdown zone marker
                         3 Fixed distance marker
                         4 Center line and rwy number
                         5 ??
                         6 ??
                         7 ??
27    1   Rwy ID; bit 6 on = L, bit 7 on = R, both on = C
28    2   Rwy edge lighting: 0 = off, 1 = on
30    2   Rwy surface: this field references the RUNWAY??.R8 textures:
          Num. 0: RUNWAY00.R8 (mud)
               1: RUNWAY01.R8 (concrete)
               2: RUNWAY02.R8 (asphalt)
               3: RUNWAY03.R8 (grass)
32    2   displaced threshold in feet (not supported?)
34    2   ???
36    2   displaced threshold in feet (not supported?)
38    2   ???
Landing end info:
40    1   Threshold lights: 0 = off, 1 = on
41    1   approach lighting: 2 = MALSR, 6 = ALSF-1, 7 = ALSF-2
42    1   number of strobes
43    1   number of VASI bars: 0 = no VASI, 1 = two bars, 2 = three bars
44    2   VASI angle in Deg16
46    2   VASI displacement from center line in feet
48    2   VASI displacement from midpoint of runway in feet
50    2   distance between each bar of VASI in feet

Takeoff end info:
52    1   threshold lights: 0 = off, 1 = on
53    1   approach lighting: 2 = MALSR, 6 = ALSF-1, 7 = ALSF-2
54    1   number of strobes
55    1   number of VASI bars: 0 = no VASI, 1 = two bars, 2 = three bars
56    2   VASI angle in Deg16
58    2   VASI displacement from center line in feet
60    2   VASI displacement from midpoint of runway in feet
62    2   distance between each bar of VASI in feet

This record draws a runway.
*/

void
anaRunwayData(char *name, u_int8 c, int len)
{
	lat_micro_t	lat;
	lon_micro_t	lon;
	alt_micro_t	alt;
	u_int16		heading;
	u_int16		length;
	u_int16		width;
	u_int8		markers;
	u_int8		rwyid;
	u_int16		rwyedge;
	u_int16		rwysurface;

	u_int16		throff_n, ext_n;
	u_int16		throff_f, ext_f;

	u_int8		threshold_n, aplight_n, numstrobe_n, numvasi_n;
	u_int16		vasiangle_n;
	s_int16		vasidcent_n, vasidmid_n, vasidist_n;

	u_int8		threshold_f, aplight_f, numstrobe_f, numvasi_f;
	u_int16		vasiangle_f;
	s_int16		vasidcent_f, vasidmid_f, vasidist_f;


	get1();
	lat.dec_part	= get2();
	lat.long_part   = get4();
	lon.dec_part	= get2();
	lon.long_part	= get4();
	alt.dec_part	= get2();
	alt.long_part	= get4();
	
	heading		= get2();
	length		= get2();
	width		= get2();
	markers		= get1();

	rwyid		= get1();
	rwyedge		= get2();
	rwysurface	= get1();	/* There are some bugs in fs5struc.txt ... ### */
	get1();

	throff_n	= get2();	/* ### maybe not correct... should be test */
	ext_n		= get2();	/* ### */
	throff_f	= get2();	/* ### */
	ext_f		= get2();	/* ### */

	/* Landing end info */
	threshold_n	= get1();
	aplight_n	= get1();
	numstrobe_n	= get1();
	numvasi_n	= get1();
	vasiangle_n	= get2();
	vasidcent_n	= - get2();	/* minus! */
	vasidmid_n	= - get2();	/* minus! */
	vasidist_n	= get2();

	/* Takeoff end info */
	threshold_f	= get1();
	aplight_f	= get1();
	numstrobe_f	= get1();
	numvasi_f	= get1();
	vasiangle_f	= get2();
	vasidcent_f	= get2();
	vasidmid_f	= get2();
	vasidist_f	= get2();

	/* \ */
	if (IS_PASS2) {		
		fprintf(outfp, "\tRunwayData( ");
		fprintf(outfp, "%s ", latstr(lat_micro2lat(&lat), NULL));
		fprintf(outfp, "%s\n", lonstr(lon_micro2lon(&lon), NULL));

		fprintf(outfp, "\t\talt\t%.2f\n", alt_micro2alt(&alt));
		fprintf(outfp, "\t\theading\t%.2f\n", deg162deg(heading) + 0.005);
		fprintf(outfp, "\t\tlength\t%d\n", length);
		fprintf(outfp, "\t\twidth\t%d\n", width);
		fprintf(outfp, "\t\tmarkers\t%02X\n", markers);		

		fprintf(outfp, "\t\tid\t%s\n", RunwayName(rwyid));

		fprintf(outfp, "\t\tlights\t%02X\n", rwyedge);
		fprintf(outfp, "\t\tsurface\t%d\n",  rwysurface);
	
		fprintf(outfp, "\t\tThrOffN\t%d\n",	throff_n);
		fprintf(outfp, "\t\tExtN\t%d\n",	ext_n);

		fprintf(outfp, "\t\tThrOffF\t%d\n",	throff_f);
		fprintf(outfp, "\t\tExtF\t%d\n",	ext_f);

		fprintf(outfp, "\t\tTHRLightsN\t%x\n",	threshold_n);
		fprintf(outfp, "\t\tAprLN\t%d\n",	aplight_n);
		fprintf(outfp, "\t\tStrobesN\t%d\n",	numstrobe_n);
		fprintf(outfp, "\t\tVasiBarsN\t%d\n",	numvasi_n);
		fprintf(outfp, "\t\tVasiSlopeN\t%.2f\n",deg162deg(vasiangle_n));
		fprintf(outfp, "\t\tVasiSideN\t%d\n",	vasidcent_n);
		fprintf(outfp, "\t\tVasiDistN\t%d\n",	vasidmid_n);
		fprintf(outfp, "\t\tVasiSpaceN\t%d\n",	vasidist_n);

		fprintf(outfp, "\t\tTHRLightsF\t%x\n",	threshold_f);
		fprintf(outfp, "\t\tAprLF\t%d\n",	aplight_f);
		fprintf(outfp, "\t\tStrobesF\t%d\n",	numstrobe_f);
		fprintf(outfp, "\t\tVasiBarsF\t%d\n",	numvasi_f);
		fprintf(outfp, "\t\tVasiSlopeF\t%.2f\n",deg162deg(vasiangle_f));
		fprintf(outfp, "\t\tVasiSideF\t%d\n",	vasidcent_f);
		fprintf(outfp, "\t\tVasiDistF\t%d\n",	vasidmid_f);
		fprintf(outfp, "\t\tVasiSpaceF\t%d\n",	vasidist_f);
		
		fprintf(outfp, "\t)\n");

		/*RunwayData( 54:11:16.2394 07:55:00
		alt        3.5  heading    27.2 id      3
		lenth      4788 width      98
		surface    2    lights     13   markers 36
		THRLightsF 1    THRLightsN 1
		VASIBarsF  1    VASISpaceF 330  VASIdistF 1115
		VASIsideF  r120 VASIslopeF 3.0
		VASIBarsN  1    VASISpaceN 330  VASIdistN 1115
		VASIsideN  L120 VASIslopeN 3
		ThrOffN    740  ThrOffF    740
	    )*/	
	}
}


/*-- 46h ------------- RotoCallInd (22 bytes)

2     2   byte offset
4     2   delta East  translation
6     2   delta Alt.  translation
8     2   delta North translation
10    2   direct rotation around E axis (Deg16)
12    2   LocSeg var with indirect rotation around E axis (Deg16)
14    2   direct rotation around N axis (Deg16)
16    2   LocSeg var with indirect rotation around N axis (Deg16)
18    2   direct rotation around A axis (Deg16)
20    2   LocSeg var with indirect rotation around A axis (Deg16)*/
void 
anaTransformCall(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset, dx, dz, dy;
	u_int16 rotx, roty, rotz, varx, vary, varz;

	basep = GetPointer() - 1;

	get1();
	offset = get2();

	dx = get2();
	dz = get2();
	dy = get2();

	rotx = get2();
	varx = get2();
	roty = get2();
	vary = get2();
	rotz = get2();
	varz = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %d %d %d\n", dx, dz, dy);
		fprintf(outfp, "\t\t%.2f %X  %.2f %X  %.2f %X )\n", 
			deg162deg(rotx), varx, deg162deg(roty), vary, deg162deg(rotz), varz);
	}
}

/*-- 49h ------------- Building (18 bytes)

1     1   [code 2nd byte]
2     2   building appearance:
          bits 0-2 are used to reference one of the SIDE?.R8 textures:
          Num. 0: SIDE1.R8 (     beige bldg with vert. strips)
               1: SIDE2.R8 (     white bldg with vert. strips)
               2: SIDE3.R8 ( dark gray bldg with vert. strips)
               3: SIDE4.R8 (      gray bldg with vert. strips)
               4: SIDE5.R8 (     white bldg with vert. black strips)
               5: SIDE6.R8 (     white bldg with horz. black strips)
               6: SIDE7.R8 (     beige bldg with horz. black strips)
               7: SIDE8.R8 (light gray bldg with vert. strips)
          bit 3: inclined roof
          bit 4: inclined sides (pyramid)
          bits 5-15: unused??
4     2   hidden side flags: each bldg side is controlled by a flag, if the
          flag is 0, the wall is not drawn:
          bit  0: roof
               1: west wall
               2: east wall
               3: south wall
               4: north side
               5-15: unused??
6     2   delta East  coord. of base centre
8     2   delta Alt.  coord. of base
10    2   delta North coord. of base centre
12    2   bldg height (in delta units * 4)
14    2   bldg N-S width (in delta units)
16    2   bldg E-W length (in delta units)*/
void
anaBuilding(char *name, u_int8 c, int len)
{
	u_int16	type, wallflags, dx, dz, dy, height, x, y;

	get1();

	type = get2();
	wallflags = get2();

	dx = get2();
	dz = get2();
	dy = get2();

	height = get2();
	x = get2();
	y = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d  %d %d %d  %X %X )\n", name,
			dx, dz, dy, height, x, y, type, wallflags);
	}
}

/*-- 50h -------------  GradColour (4 bytes)

2     1   colour
1     1   ???*/
void
anaColor(char *name, u_int8 c, int len)
{
	u_int8 c1, c2;

	get1();
	c1 = get1();
	c2 = get1();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %02X %02X )\n", name, c1, c2);
	}
}

/*-- 54h -------------  CallAsm (4 bytes)

2     2   offset
4     2   arg
*/
void
anaCallAsm(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 arg;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	arg = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %X )\n", arg);
	}


}

/*-- 55h -------------  SurfaceType (10 bytes)
 
2	2	SurfaceType ?
4	2	X width		?
6	2	Y width		?
8	2	altitude	?
*/
void
anaSurfaceType(char *name, u_int8 c, int len)
{
	u_int16	type;
	s_int16	width_x, width_y, alt;

	get1();
	type = get2();
	width_x = get2();
	width_y = get2();
	alt = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d %d )\n", name, type, width_x, width_y, alt);
	}
}

/* -- 58h -------------  MoveTexture (18 bytes)

2     2   x1
4     2   ???
6     2   y1
8     2   ???
10    2   x2
12    2   ???
14    2   y2
*/
void
anaMoveTexture(char *name, u_int8 c, int len)
{
	s_int16	x1, y1, x2, y2;

	get1();
	get2();
	x1 = get2();
	get2();
	y1 = get2();
	get2();
	x2 = get2();
	get2();
	y2 = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d %d )\n", name, x1, y1, x2, y2);
	}
}

/* -- 5Dh -------------  RepeatBitmap (8 bytes)

2     2   dx
2     2   dz
2     2   dy
*/
void anaRepeatBitmap(char *name, u_int8 c, int len)
{
	s_int16 dx, dz, dy;

	get1();
	dx = get2();
	dz = get2();
	dy = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, dx, dz, dy);
	}
}


/*-- 65h -------------  SurfColInd (4 bytes)

2     2   LocSeg var address	*/
void
anaLoadColor(char *name, u_int8 c, int len)
{
	u_int16 var;

	get1();
	var = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %X )\n", name, var);
	}
}

/*-- 68h ------------- TextureShading (8 bytes)

2     2   vector East  comp.
4     2   vector Alt.  comp.
6     2   vector North comp.*/

void
anaTexPolyShading(char *name, u_int8 c, int len)
{
	s_int16 dx, dz, dy;

	get1();
	dx = get2();
	dz = get2();
	dy = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, dx, dz, dy);
	}
}

/*-- 69h -------------  HighwayMoveTo (10 bytes)

2     2   half width in delta units
4     2   delta East coordinate
6     2   delta Altit. coordinate
8     2   delta North coordinate*/
void
anaRoadMoveTo(char *name, u_int8 c, int len)
{
	s_int16 width, dx, dz, dy;

	get1();
	width = get2();
	dx = get2();
	dz = get2();
	dy = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d  %d %d %d )\n", name, width, dx, dz, dy);
	}
}

/*-- 6Ah -------------  HighwayLineTo (8 bytes)

2     2   delta East coordinate
4     2   delta Altit. coordinate
6     2   delta North coordinate*/
void
anaRoadLineTo(char *name, u_int8 c, int len)
{
	s_int16 dx, dz, dy;

	get1();
	dx = get2();
	dz = get2();
	dy = get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %d %d %d )\n", name, dx, dz, dy);
	}
}

/*-- 6Dh ------------- IfHRes (8 bytes)


IfHRes( :Label h s )
        -
        The horizontal size/resolution is tested.
        -
        h       decimal, integer
        s       decimal, integer
*/

void
anaIfVHRes(char *name, u_int8 c, int len)
{
	s_int16 offset;
	s_int16 h, s;
	u_int32 basep;

	basep = GetPointer() - 1;

	get1(); /* dummy */
	offset = get2();

	h = get2();
	s = get2();
	
	if (IS_PASS1) {
		PutLabel(basep + offset);
	}

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %d %d )\n", h, s);
	}
}

/*-- 72h ------------- EndLights (20 bytes)

1     1   [code 2nd byte]
2     2   landing course (Deg16)
4     2   threshold light width (delta)
6     2   threshold light distance from ref. point (delta u).  A positive
          distance means the lights are 'beyond' the ref. point when looking
          in the landing course; a negative distance means the lights are
          'before' the ref. point
8     1   Threshold lights: 0 = off, 1 = on
9     1   approach lighting: 2 = MALSR, 6 = ALSF-1, 7 = ALSF-2
10    1   number of strobes
11    1   number of VASI bars: 0 = no VASI, 1 = two bars, 2 = three bars
12    2   VASI angle in Deg16
14    2   VASI displacement from center line in feet
16    2   VASI displacement from midpoint of runway in feet
18    2   ???
*/
void
anaRunwayLights(char *name, u_int8 c, int len)
{
	u_int16		heading;
	u_int16		length;
	u_int16		width;

	u_int8		threshold, aplight, numstrobe, numvasi;
	u_int16		vasiangle;
	s_int16		vasidcent, vasidmid, vasidist;

	get1();

	heading = get2();
	width	= get2();
	length	= get2();
	
	threshold	= get1();
	aplight		= get1();
	numstrobe	= get1();
	numvasi		= get1();
	vasiangle	= get2();
	vasidcent	= get2();
	vasidmid	= get2();
	vasidist	= get2();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s(\n", name);
		fprintf(outfp, "\t\theading\t%.2f\n",	deg162deg(heading));
		fprintf(outfp, "\t\tlength\t%d\n",		length);
		fprintf(outfp, "\t\twidth\t%d\n",		width);

		fprintf(outfp, "\t\tTHRLights\t%02x\n",	threshold);
		fprintf(outfp, "\t\tAprL\t%d\n",		aplight);
		fprintf(outfp, "\t\tStrobes\t%d\n",		numstrobe);
		fprintf(outfp, "\t\tVasiBars\t%d\n",	numvasi);
		fprintf(outfp, "\t\tVasiSlope\t%.2f\n",	deg162deg(vasiangle));
		fprintf(outfp, "\t\tVasiSide\t%d\n",	vasidcent);
		fprintf(outfp, "\t\tVasiDist\t%d\n",	vasidmid);
		fprintf(outfp, "\t\tVasiSpace\t%d\n",	vasidist);
		fprintf(outfp, "\t\t)\n");
	}
}


/*-- 73h ------------- JumpOnCude (16 bytes)

2     2   byte offset
4     2   min East (delta)
6     2   max East (delta)
8     2   min Alt. (delta)
10    2   max Alt. (delta)
12    2   min North (delta)
14    2   max North (delta)*/
void
anaMonitor3D(char *name, u_int8 c, int len)
{
	u_int32 basep;
	u_int16 offset;
	s_int16 minx, maxx, minz, maxz, miny, maxy;

	basep = GetPointer() - 1;

	get1();/*dummy*/
	offset = get2();
	minx = get2();
	maxx = get2();
	minz = get2();
	maxz = get2();
	miny = get2();
	maxy = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\tMonitor3D( ");
		PrintLabel(basep + offset);
		fprintf(outfp, " %d %d  %d %d  %d %d )\n",
			minx, maxx, minz, maxz, miny, maxy);
	}
}

/*-- 74h -------------  AddToQueue (6 bytes)

2     2   byte offset
4     2   queue index (0 - 63)*/
void 
anaLayerCall(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16	layer;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	layer = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %d )\n", layer);
	}
}


/*-- 77h -------------  RefPoint77 (32 bytes)

1     1   [code 2nd byte]
2     2   byte offset (???)
4     2   ??? visibility related
6     2   ??? invisibility cone
8     2   ???
10    4   Scale factor
14    6   Latitude in Mu
20    6   Longitude in Mu
26    6   MSL Altitude in Mu (ignored)
*/
void
anaRefPoint(char *name, u_int8 c, int len)
{
	u_int32 basep;

	char type;
	s_int16 offset;
	u_int32	scale;
	lat_micro_t	lat;
	lon_micro_t	lon;
	alt_micro_t	alt;
	u_int16 v1, v2, v3;

	basep = GetPointer() - 1;

	switch (c) {
	case 0x2f:
		type = '2';
		break;

	case 0x77:
		type = '7';
		break;
	}

	get1();
	offset = get2();	/* jump offset */
	v1 = get2();
	v2 = get2();
	v3 = get2();
	scale = get4();
	lat.dec_part  = get2();
	lat.long_part = get4();
	lon.dec_part  = get2();
	lon.long_part = get4();
	alt.dec_part  = get2();
	alt.long_part = get4();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\tRefPoint( %c ", type);
		PrintLabel(basep + offset);
		if (scale == 0) scale = 1; /* ad hoc! */
		fprintf(outfp, " %f ", 65536.0 / (double)scale);
		fprintf(outfp, "%s ", latstr(lat_micro2lat(&lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_micro2lon(&lon), NULL));
		if (type == '2') {
			fprintf(outfp, "E= %f ", alt.long_part + alt.dec_part / 65536.0);
		}
		fprintf(outfp, "v1= %d v2= %d )\n", v1, v2);
	}
}
  
/*
-- 3Eh -------------  Polygon (14 bytes + 2*NumOfVertices)

2     2   number of vertices
4     2   vector East component
6     2   vector Alt. component
8     2   vector North component
10    4   distance from the ref. point (in delta units * vector modulus)
*/
void
ana3DPoly(char *name, u_int8 c, int len)
{
	int numvert, i;
	s_int16	vx, vz, vy;
	int number;
	double dist;

	get1();	/* align */
	numvert = get2();
	vx = get2();
	vz = get2();
	vy = get2();
	dist = (double)((s_int32)get4()) / 32767.0;		/* ### ??? */

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( m %d %d %d %f   ", name, vx, vz, vy, dist);

		for (i=0; i<numvert; i++) {
			number = get2();
			fprintf(outfp, "%d ", number);
		}
		fprintf(outfp, ")\n");
	} else {
		ProcPointer(2 * numvert);
	}
}


/*
-- 70h -------------  Polyline (4 bytes + 4*NumberOoutfpoints)

2     2   offset
2     2   number of points

then for each point:

x     2   delta East coordinate
x+2   2   delta North coordinate
*/
void
anaSenseBorder(char *name, u_int8 c, int len)
{
	u_int32	basep;

	int numvert, i;
	s_int16 offset;
	s_int16 x, y;

	basep = GetPointer() - 1;

	get1();/* align */
	offset = get2();
	numvert = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
		ProcPointer(4 * numvert);
	} else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, "\n");

		for (i=0; i<numvert; i++) {
			x = get2();
			y = get2();
			fprintf(outfp, "\t\t%d %d\n", x, y);
		}
		fprintf(outfp, "\t)\n");
	}
}



/*
-- 62h -------------  MonitorPt (6 bytes + 2*NoOfPoints)

2     2   ???
4     2   number of points
*/
void
anaMonitorPt(char *name, u_int8 c, int len)
{
	unsigned int	i;
	u_int8 d1;
	s_int16 offset;
	u_int16 numvert, n;
	u_int32 basep;

	basep = GetPointer() - 1;

	d1 = get1();
	offset = get2();
	numvert = get2();
	
	if (IS_PASS1) {
		PutLabel(basep + offset);
		ProcPointer(2 * numvert);
	} else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, "\n\t\t");

		for (i = 0; i<numvert; i++) {
			n = get2();
			fprintf(outfp, "%d ", n);
		}
		fprintf(outfp, ")\n");
	}
}


/*
 * -- 75h -------------- PerspectiveSurface   (by Jim Reuter)
 * PerspectiveSurface call, one label parameter.
 */
void
anaPerspectiveSurface( char *name, u_int8 c, int len )
{
	u_int32 basep;
	s_int16 offset;

	basep = GetPointer() - 1;

	get1();
	offset = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t;;; %s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " ) - record unsupported\n");

		fprintf(outfp, "\tDbx( %X 0 )\n", c);
		fprintf(outfp, "\tDwx( %X )\n", offset );
	}
}


/*-- 81h -------------  ShadowPos (SDK: SHADOW_POSITION)

1     1   [code 2nd byte]
2     6   Latitude
8     6   Longtitude
14    6   Altitude
*/
void
anaShadowPos(char *name, u_int8 c, int len)
{
	u_int32 basep;

	lat_micro_t	lat;
	lon_micro_t	lon;
	alt_micro_t	alt;

	lat.dec_part  = get2();
	lat.long_part = get4();
	lon.dec_part  = get2();
	lon.long_part = get4();
	alt.dec_part  = get2();
	alt.long_part = get4();

	if (IS_PASS2) {
		fprintf(outfp, "\t%s( %c ", name);
		fprintf(outfp, "%s ", latstr(lat_micro2lat(&lat), NULL));
		fprintf(outfp, "%s ", lonstr(lon_micro2lon(&lon), NULL));
		fprintf(outfp, "%f )\n", alt.long_part + alt.dec_part / 65536.0);
	}
}

/*
 * -- 83h ----------- SetScale (by Jim Reuter)
 * Format is:
 *  label
 *  word
 *  word
 *  float
 */
void
anaSetScale(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset;
	u_int16 i1, i2;
	double f;
	u_int32 intfl;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	i1 = get2();
	i2 = get2();
	intfl = get4();
	f = 65536.0 / (double)(intfl);


	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %d %d %f )\n", i1, i2, f);
	}
}

/*
 * Call DLL - 7 bytes + record length
 * Record 8Ch                         
 * Calls a routine found in FSASMLIB.DLL by Mr. Szofran 
 * This is a dynamic record, with the record length stored in the 7th
 * byte
 */
/* by Alessandro Antonini */

void anaCallDll(char *name, u_int8 c, int len)
{
	u_int8 reclen;		/* is it really 1 byte? seems so, since it works */
	int i;
	u_int8 number;

	get1();			/* synchronize */

	if (IS_PASS2) {
		fprintf(outfp,"; SubRoutine Call from FSASMLIB.DLL\n");
		fprintf(outfp,"\tDbx( %X 0 )\n", c);
	}

	/* Reads and prints 6 bytes before reclen */
	if (IS_PASS2) {
		fprintf(outfp, "\tDbx( ");
	}
	for( i=0; i<6; i++ ) {
		number = get1();
		if (IS_PASS2) {
			fprintf(outfp,"%X ", number);
		}
	}
	if (IS_PASS2) {
		fprintf(outfp, ")\n");
	}

	/* Get Length */
	reclen = get1();
	if (IS_PASS2) {
		fprintf(outfp, "\tDbx( %X )\n", reclen);
		fprintf(outfp, "\tDbx( ");
	}

	/* now decode the rest */
	for (i=0; i<reclen-3; i++ ) {
		number=get1();
		if (IS_PASS2) {
			fprintf(outfp,"%X ", number);
		}
	}
	if (IS_PASS2) {
		fprintf(outfp, ")\n");
	}
}

/***** New Commands *******/

/*
 * Following commands group are recently added to SCASM.
 */

void anaPitchCall(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int16 offset, angle;

	basep = GetPointer() - 1;

	get1();
	offset = get2();
	angle = get2();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " %x ", angle);
		fprintf(outfp, " )\n");
	}
}


/*-- 8Ah -------------  Call32 (6 bytes)

2     4   offset
*/
void 
anaCall32(char *name, u_int8 c, int len)
{
	u_int32 basep;
	s_int32 offset;

	basep = GetPointer() - 1;

	get1();
	offset = get4();

	if (IS_PASS1) {
		PutLabel(basep + offset);
	}
	else {
		fprintf(outfp, "\t%s( ", name);
		PrintLabel(basep + offset);
		fprintf(outfp, " )\n");
	}
}
