/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: sec9tab.c,v 1.6 2000/01/04 06:42:25 tmurakam Exp $
 */
/*
 * section9 analyze table
 */

#include "type.h"
#include "sec9ana.h"

SecRecordTab	sec9tab[] = {
	/*0x00   ExitSDL */				/* SDK: EOF */	
	{ 0x01, "Forever",	anaForever,	-1 },	/* SDK: LOGOL */
	{ 0x02, "Nop",		anaNop,		-1 },	/* SDK: NOOP */
	{ 0x03, "Switch",	anaSwitch,	-1 },	/* SDK: CASE */
	{ 0x04, "NopBX",	anaNopBX,	-1 },	/* SDK: DEBUG */
	{ 0x05,	"StartSurface", anaNoArg,	-1 },	/* SDK: SURFACE */
	{ 0x06, "MoveTo",	anaXTo,		-1 },	/* SDK: SPNT */
	{ 0x07, "DrawTo",	anaXTo,		-1 },	/* SDK: CPNT */
	{ 0x08,	"EndSurface",	anaNoArg,	-1 },	/* SDK: CLOSURE */ 
	{ 0x09, "StartDrawing",	anaDump,	2 },	/* SDK: GSURF */
	{ 0x0A, "MoveTo",	anaXTo10,	-1 },	/* SDK: GSPNT */
	{ 0x0B, "DrawTo",	anaXTo10,	-1 },	/* SDK: GCPNT */
	{ 0x0C, "CloseDrawing",	anaDump,	2 },	/* SDK: GCLOSURE */
	{ 0x0D,	"Jump", 	anaJumpCall,	-1 },	/* SDK: JUMP */
	{ 0x0E, "DefPoint",	anaDefPoint,	-1 },	/* SDK: DEFRES */
	{ 0x0F, "MoveToPt",	anaXToPt,	-1 },	/* SDK: STRRES */
	{ 0x10, "DrawToPt",	anaXToPt,	-1 },	/* SDK: CNTRES */
	{ 0x11, "DefPPoint",	anaDefPPoint,	-1 },	/* SDK: GDEFRES */	
	{ 0x12, "MoveToPPt",	anaXToPPt,	-1 },	/* SDK: GSTRRES */
	{ 0x13, "DrawToPPt",	anaXToPPt,	-1 },	/* SDK: GCNTRES */
	{ 0x14, "SurfaceColer",	anaColor,	-1 },	/* SDK: SETCOLOR */
	{ 0x15, "TexRelief",	anaTexRelief,	-1 },	/* SDK: ELEVATION_MAP */
	{ 0x16, "SetVar",	anaSetVar,	4 },	/* ### */
	{ 0x17,	"Unknown",	anaDump,	4 },	/* SDK: TEXTURE_ENABLE *//*###*/
	{ 0x18, "Bitmap",	anaBitmap,	-1 },	/* SDK: TEXTURE */
	{ 0x19, "Palette",	anaPalette,	-1 },	/* SDK: PALETTE */
	{ 0x1A, "Points",	anaPoints,	-1 },	/* SDK: RESLIST */
	{ 0x1B, "MonitorTr",	anaMonitorTr,	-1 },	/* SDK: IFIN_BOX_PLANE */
	{ 0x1C, "IfVarRange2",	anaIfVarRange2,	-1 },	/* SDK: IFIN2 */
	{ 0x1D, "VecPoly", 	anaVecPoly,	-1 },	/* SDK: FACE3,4,5,6,7,8,FACE *//*###*/
	{ 0x1E, "BitmapMode",	anaArgWd,	-1 },	/* SDK: HAZE */
	{ 0x1F, "Set19B2",	anaDump,	4 },	/* SDK: HORIZON */
	{ 0x20, "TexPoly",	anaTexPoly,	-1 },	/* SDK: FACET3,4_TMAP */
	{ 0x21, "IfVarRange3",	anaIfVarRange3,	-1 },	/* SDK: SUPERMESH?, IFIN3? */
	{ 0x22, "Return", 	anaNoArg,	-1 },	/* SDK: BGL_RETURN */
	{ 0x23, "Call",		anaJumpCall,	-1 },	/* SDK: BGL_CALL */
	{ 0x24, "IfVarRange",	anaIfVarRange,  -1 },	/* SDK: IFIN1 (SIDE) */
	{ 0x25, "VectorJump",	anaVectorJump,	-1 },	/* SDK: SEPARATION_PLANE (IFSIDE) */
	{ 0x26, "SetVar",	anaSetVar,	4 },	/* SDK: SETWRD */
	{ 0x27, "TexWindow",	ana3DPoly,	-1 },	/* SDK: BGL_TEXTURED_FACET3,4 */
	{ 0x28,	"BigDot",	anaBigDot,	-1 },	/* SDK: BALL */
	{ 0x29, "VecPoints",	anaVecPoints,	-1 },	/* SDK: GRESLIST */
	{ 0x2A, "ShadedPoly",	ana3DPoly,	-1 },	/* SDK: GFACETx */
	{ 0x2B, "Nop2B",	anaNop,		-1 },	/* SDK: ADDOBJ_32 */
	{ 0x2C, "VectorJump",	anaVectorJump,	-1 },	/* SDK: REJECT *//* ### */
	{ 0x2D, "Col2D",	anaDump,	6 },	/* SDK: SCOLOR24 */
	{ 0x2E, "Col2E",	anaDump,	6 },	/* SDK: LCOLOR24 */
	{ 0x2F, "RefPoints",	anaRefPoint,	-1 },	/* SDK: SCALE */
	{ 0x30, "Brightness",	anaBrightness,	4 },	/* SDK: SET_BRIGHTNESS */
	{ 0x31, "AutoPoints",	anaAutoPoints,	-1 },	/* SDK: RESROW */
	{ 0x32, "PerspectiveCall",anaJumpCall, -1 },	/* SDK: ADDOBJ */
	{ 0x33, "RotatedCall",	anaRotatedCall, -1 },	/* SDK: INSTANCE_CALL */ 
	{ 0x34, "SetScaleX",	anaSetScaleX,	-1 },	/* SDK: SUPER_SCALE */
	{ 0x35, "DotLine",	anaDotLine,	-1 },	/* SDK: PNTROW */
	{ 0x36, "PatternLine",	anaUnknown,	1 },	/* ### valiable length, should be skipped */
	{ 0x37, "Dot",		anaDot,		-1 },	/* SDK: PNT */
	{ 0x38, "ConcavePoly",	anaNoArg,	-1 },	/* SDK: CONCAVE */
	{ 0x39, "IfVarAnd",	anaIfVarAnd,	-1 },	/* SDK: IFMSK */
	{ 0x3A, "CondNSRefPointInd",anaUnknown, 12 },	/* SDK: VPOSITION *//* ### */
	{ 0x3B, "PitchCall",	anaPitchCall,	-1 },	/* SDK: VINSTANCE_CALL *//* (old:Call3B) ### */
	{ 0x3C, "CondNSRefPoint",anaUnknown,	28 },	/* SDK: POSITION *//* ### */
	{ 0x3D, "Synth",	anaUnknown,	24 },	/* SDK: SEED */
	{ 0x3E, "Poly",		ana3DPoly,	-1 },	/* SDK: FACETx */
	{ 0x3F, "ShadowCall",	anaJumpCall,	-1 },	/* SDK: SHADOW_CALL */

	{ 0x40, "ShadowPosInd",	anaArgWx,	-1 },	/* SDK: SHADOW_VPOSITION */
	{ 0x41, "ShadowCallVI",	anaShadowCallVI,-1 },	/* SDK: SHADOW_VICALL *//* FS98 ### */
	{ 0x42, "Runway",	anaRunwayData,	-1 },	/* same as 0x44 */
	{ 0x43, "Nop43",	anaNop,		-1 },
	{ 0x44, "Runway",	anaRunwayData,	-1 },	/* SDK: TEXTURE_RUNWAY or RUNWAY_MANI or POLYGON_RUNWAY *//* ThrOff, ExtN??? ### */
	{ 0x45, "DotLine",	anaDotLine,	-1 },		/* something wrong ... */
	{ 0x46, "TransformCall",anaTransformCall,-1 },	/* SDK: POINT_VICALL (PVICALL) */
	{ 0x47, "Rec47",	anaUnknown,	4 },	/* SDK: MAP_SCALE *//* ### */
	{ 0x48, "SetLocSeg",	anaDump,	4 },	/* SDK: VAR_SEG *//* ### */
	{ 0x49, "Building",	anaBuilding,	-1 },	/* SDK: BLDING */
	{ 0x4A, "CondCall1A",	anaUnknown,	12 },	/* SDK: LANDING_LIGHTS_VICALL *//* ### */
	{ 0x4B, "Rec4B",	anaDump,	4 },	/* SDK: BGL_OVERLAY *//* ### */
	{ 0x4C, "CondRefPointInd", anaDump,	16},	/* SDK: VSCALE *//* ### replace by 2F ? */
	{ 0x4D, "EquVarToLocVar",  anaDump,	6},	/* SDK: VAR2LOW64K */
	{ 0x4E, "EquLocVarToVar",  anaDump,	6},	/* SDK: LOW64K2VAR */
	{ 0x4F, "EquVar",	anaDump,	6},	/* SDK: MOVWRD */
	{ 0x50, "ShadedColor",	anaColor,	-1 },	/* SDK: GCOLOR */
	{ 0x51, "LineColor",	anaColor,	-1 },	/* SDK: LCOLOR */
	{ 0x52, "SurfaceColor",	anaColor,	-1 },	/* SDK: SCOLOR */
	{ 0x53, "Color53",	anaDump,	4 },	/* SDK: GLOLOR_ABS */
	{ 0x54, "CallAsm",	anaCallAsm,	6 },
	{ 0x55, "SurfaceType",	anaSurfaceType,	-1 },	/* SDK: SET_SURFACE_TYPE */
	{ 0x56,	"Jump",		anaJumpCall,	-1 },	/* SDK: SET_WEATHER *//* something wrong... ### */
	{ 0x57, "WEATHER",	anaDump,	-1 },	/* SDK: WEATHER *//*###*/ 
	{ 0x58, "MoveTexture",	anaMoveTexture,	-1 },	/* SDK: TEXTURE_BOUNDS */
	{ 0x59, "Rec59",	anaDump,	4 },	/* SDK: VAR_SEG_ID */
	{ 0x5A, "Call",		anaJumpCall,	-1 },	/* SDK: SEED_ADDOBJ *//* something wrong... ### */
	{ 0x5B, "CallInd",	anaDump,	4 },	/* SDK: INDIRECT_CALL */
	{ 0x5C, "FarCall",	anaDump,	6 },	/* SDK: BGL_FAR_CALL */
	{ 0x5D, "RepeatBitmap",	anaRepeatBitmap, -1 },	/* SDK: TEXTURE_REPEAT */
	{ 0x5E, "Rec5E",	anaDump,	4 },	/* SDK: TEXTURE_ROTATE */
	{ 0x5F, "IfVRes",	anaIfVHRes,	-1 },	/* SDK: IFSIZEV */
	/*{ 0x60, "Polygon60",	???,		? },*/	/* SDK: FACEx_TMAP *//* ### */
	{ 0x61,	"Nop6",		anaDump,	6 },	/* SDK: RESLIST_SCALE */
	{ 0x62, "MonitorPt",	anaMonitorPt, -1 },	/* SDK: IFVISx */
	{ 0x63, "Rec63",	anaUnknown,	10 },	/* SDK: BGL_LIBRARY(_CALL) */
	{ 0x64,	"Nop64",	anaDump,	2 },	/* SDK: SYNTH_BUILDINGS */
	{ 0x65, "LoadSurfaceColor", anaLoadColor, -1},	/* SDK: VSCOLOR */
	{ 0x66, "LoadShadedColor",  anaLoadColor, -1},	/* SDK: VGCOLOR */
	{ 0x67, "LoadLineColor",  anaLoadColor,	-1},	/* SDK: VLCOLOR */
	{ 0x68, "TexPolyShading",anaTexPolyShading,-1},	/* SDK: TMAP_LIGHT_SHADE */
	{ 0x69, "RoadMoveTo",	anaRoadMoveTo,  -1 },	/* SDK: ROAD_START */
	{ 0x6A, "RoadLineTo",	anaRoadLineTo,	-1 },	/* SDK: ROAD_CONT */
	{ 0x6B, "RoadMoveTo2",	anaRoadMoveTo,  -1 },	/* SDK: RIVER_START */
	{ 0x6C, "RoadLineTo2",	anaRoadLineTo,	-1 },	/* SDK: RIVER_CONT */
	{ 0x6D, "IfHRes",	anaIfVHRes,	-1 },	/* SDK: IFSIZEH */
	{ 0x6E, "TaxiMoveTo",	anaRoadMoveTo,	-1 },	/* SDK: TAXIWAY_START */
	{ 0x6F, "TaxiLineTo",	anaRoadLineTo,	-1 },	/* SDK: TAXIWAY_CONT */
	{ 0x70,	"SenseBorder",	anaSenseBorder, -1 },	/* SDK: AREA_SENSEx *//* Something wrong. Check offset analyzation! ### */
	{ 0x71, "Rec71",	anaUnknown,	4 },	/* SDK: ALTITUDE_SET */
	{ 0x72, "RunwayLights",	anaRunwayLights, -1},	/* SDK: APPROACH_LIGHTS */
	{ 0x73, "Monitor3D",	anaMonitor3D,	-1 },	/* SDK: IFINBOXP */
	{ 0x74, "LayerCall",	anaLayerCall,	-1 },	/* SDK: ADDCAT */
	{ 0x75, "PerspectiveSurface",anaPerspectiveSurface,-1 }, /* SDK: ADDMTN */
	{ 0x76, "Perspective",	anaNoArg,	-1 },
	{ 0x77, "RefPoint",	anaRefPoint, -1 },	/* SDK: SCALE_AGL */
	{ 0x78, "HighwayWLineTo",anaDump,	10 },	/* SDK: ROAD_CONTW */
	{ 0x79, "HighwayWLineTo79",anaDump,	10 },	/* SDK: RIVER_CONTW */
	{ 0x7A,	"ShadedTexPoly", anaTexPoly, -1},	/* SDK: GFACETx_TMAP *//* something wrong... ### */
	/*{ 0x7B, "ShadedTexPoly7B", ???, ??? },*/ /* SDK: GFACE4_TMAP *//* ad hoc ### */
	{ 0x7C, "Error7C",	anaDump,	2 },	/* SDK: POLYLINE_FACET */
	{ 0x7D, "Inst_7D",	anaNoArg,	2 },	/* SDK: PERSPECTIVE */
	{ 0x7E, "SetVar7E",	anaSetVar,	4 },	/* SDK: SETWORD_LOW64K */
	{ 0x7F, "City",		anaNoArg,	2 },	/* SDK: CITY */
	{ 0x80, "DotPt",	anaArgWd,	-1 },	/* SDK: RESPNT */
	{ 0x81, "Smoothing",	anaArgWd,	-1 },	/* SDK: ANTI_ALIAS */
	{ 0x82, "ShadowPos", 	anaShadowPos,	-1 },	/* SDK: SHADOW_POSITION */
	{ 0x83, "SetScale",	anaSetScale,	-1 },	/* SDK: RESCALE */
	{ 0x84, "Rec84",	anaDump,	10 },	/* SDK: SURFACE_NORMAL */
	/*{ 0x87, "SetPalette1", ???, ??? }, */		/* SDK: NIGHT_COLOR_RANGE */
	{ 0x8A, "Call32",	anaCall32,	-1 },	/* SDK: BGL_CALL_32 */
	{ 0x8B, "ADDCAT_32",	anaDump,	-1 },	/* SDK: ADDCAT_32 */
	{ 0x8C, "CallDll",	anaCallDll,	-1 },	/* SDK: ASMCALL_32 */
	{ 0x8D, "FILE_MARKER_32",anaDump,	-1 },	/* SDK: FILE_MARKER_32 */	  
	{ 0x8E, "VFILE_MARKER"	,anaDump,	-1 },	/* SDK: VFILE_MARKER */	  
	{ 0x8F, "Transparency",	anaArgLx,	-1 },	/* SDK: ALPHA */
	{ 0x90, "PolyM",	ana3DPoly,	-1},	/* SDK: unknown */
	{0,	"DUMMY",		0 }
};
