/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: synthana.c,v 1.4 2000/01/04 06:42:28 tmurakam Exp $
 */
/*
 * Section 1-6 (Synthetic Scnery) analyze
 */

#include <stdio.h>
#include <stdlib.h>

#include "type.h"
#include "util.h"
#include "conv.h"
#include "synthana.h"

/* prototypes */
void AnalyzeSynthMain(void);
void AnalyzeSingleBlock(void);
void AnalyzeMultiBlock(void);

extern void
AnalyzeSynth(u_int32 sec_pointer, int secno)
{
	u_int8 c;
	u_int32	offset;
	u_int32 posbak;
	char fname[30];
	u_int16 lat; 
	char max_north[LLBLEN], min_north[LLBLEN];

	if (secno < 1 || secno > 6) {
		return;	/* invalid section number */
	}

	sprintf(fname, "%s%d", SECFILENAME_PRE, secno);
	OpenOutFile(fname, TRUE);

	/* Write Header and LatRange */
	WriteHeader(secno);
	latstr(lat_long2lat(Header.max_north), max_north);
	latstr(lat_long2lat(Header.min_north), min_north);
	fprintf(outfp, "LatRange( %s %s )\n", min_north, max_north);

	/* Move pointer */
	MovePointer(sec_pointer);

	SET_PASS2;	/* NO PASS 1 */

	/* Analyze Latitude Table */
	while (1) {
		c = get1();
		if (c == 0) break;	/* end of table */

		if (c != 0x16) {
			fprintf(stderr, "unknown record\n");
			exit(1);
		}

		/* 0x16: Latitude Table Pointer */
		lat = get2();
		offset = get4();
		fprintf(outfp, "Synth %d %d\n", secno, lat);

		posbak = GetPointer();
		MovePointer(sec_pointer + offset);
		AnalyzeSynthMain();

		MovePointer(posbak);
	}

	CloseOutFile();

}

void AnalyzeSynthMain(void)
{
	u_int8	c;

	while (1) {
		c = get1();
		if (c == 0) break;

		switch(c) {
		case 0x04:
			AnalyzeSingleBlock();
			break;
		
		case 0x05:
			AnalyzeMultiBlock();
			break;

		default:
			fprintf(stderr, "unknown record\n");
			exit(1);
		}
	}
}

/*
-- 04h ------------- Single block group (10 bytes; source RAYMOND DAVEY)

1     2   Longitude in long. units (see above)
3     1   selector 1
4     1   selector 2
5     2   object kind (see below):
               0000h ?
               1111h terrain
               2222h mountain
               3333h coastline
               4444h coastline
7     1   MSL Altitude in 1/256m
8     2   MSL Altitude in m
*/

void AnalyzeSingleBlock(void)
{
	u_int16	lon;
	u_int8	sel1, sel2;
	u_int16 kind;
	u_int8 altf;
	u_int16	alt;

	lon = get2();
	sel1 = get1();
	sel2 = get1();
	kind = get2();
	altf = get1();
	alt = get2();

	fprintf(outfp, "\tBlock %d %X %X %X %f\n", lon, sel1, sel2, kind, alt + altf / 256.0);
}

/*
-- 05h ------------- Multiple block groups (5 bytes + 7*NoOfGroups)

1     2   First Longitude in size units
3     2   Last  Longitude in size units

Then, for each Longitude value from First to Last, is added the structure:

5     1   selector 1
6     1   selector 2
7     2   object kind
9     1   MSL Altitude in 1/256m
10    2   MSL Altitude in m
*/

void AnalyzeMultiBlock(void)
{
	u_int16	lon1, lon2, lon;
	u_int8	sel1, sel2;
	u_int16 kind;
	u_int8 altf;
	u_int16	alt;

	lon1 = get2();
	lon2 = get2();

	if (lon2 < lon1) {
		fprintf(stderr, "illegal longitude in synth block\n");
		exit(1);
	}

	for (lon = lon1; lon <= lon2; lon++) {
		sel1 = get1();
		sel2 = get1();
		kind = get2();
		altf = get1();
		alt = get2();

		fprintf(outfp, "\tBlock %d %X %X %X %f\n", lon, sel1, sel2, kind, alt + altf / 256.0);
	}
}
