/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: type.h,v 1.8 2000/01/15 07:06:50 tmurakam Exp $
 */
/* 
 * type.h 
 */

#define	LITTLE_ENDIAN	/* little endian (Intel x86 and so on) */
/*efine BIG_ENDIAN*/	/* big endian */

/*
 * Constants
 */

#define	VERSION			"ver 1.9"
#define	HEADER_FILE		"scdhead.txt"
#define SECFILENAME_PRE		"scd"
#define COMPILEBAT		"compall.bat"
#define SCLINKFILE		"sclink.ctl"
#define	SCMEXT			".scm"
#define	BGLEXT			".bgl"
#ifdef	WIN32
#define	PATH_DELIMITER	'\\'	/* windows & dos */
#else	
#define	PATH_DELIMITER	'/'	/* unix */
#endif

#define	TRUE		1
#define	FALSE		0

#define LLBLEN		50		/* Lat Lon str buf len */

typedef unsigned char	u_int8;
typedef unsigned short	u_int16;
typedef unsigned long	u_int32;
typedef unsigned int	u_int;

typedef signed char	s_int8;
typedef signed short	s_int16;
typedef signed long	s_int32;
typedef signed int	s_int;

/* Latitude */
typedef s_int32		lat_int_t;	/* Integral (2m unit) : Iu (3bytes) */
typedef s_int32		lat_dec_t;	/* Decimal (1/128m unit) : Du (?) */
typedef s_int32		lat_long_t;	/* Long  (1m unit) : Lm (4bytes) */
typedef struct {
	lat_long_t	long_part;	/* 1m unit */
	u_int16		dec_part;	/* 1/65536m unit */
} lat_micro_t;				/* Mu (6bytes) */
typedef s_int16		lat_delta_t;	/* delta (2bytes) */

/* Longtitude */
typedef s_int32		lon_int_t;	/* Integral (0.96 sec. unit) (3bytes) */
typedef s_int32		lon_dec_t;	/* Decimal ( 1/256 of Integral) (2bytes) */
typedef struct {
	lat_long_t	long_part;	/* ??? */
	u_int16		dec_part;	/* 1/65536 of Decimal */
} lon_micro_t;
typedef s_int16		lon_delta_t;	/* delta */

/* Altitude */
typedef	s_int16		alt_t;		/* 1m unit */
typedef struct {
	s_int32		long_part;	/* 1m unit */
	u_int16		dec_part;	/* 1/65536m unit */
} alt_micro_t;
typedef s_int16		alt_delta_t;


/***********************
     structures
************************/

/* BGL header */
struct HeaderInfo {
	u_int16		scenery_id;
	lat_long_t	max_north;
	lat_long_t	min_north;
	lon_dec_t	max_east;
	lon_dec_t	min_east;
	u_int16		min_freq;
	u_int16		max_freq;

	u_int32		sec_pointer[22];
	u_int16		val[9];
};

extern struct HeaderInfo Header;

/* LatRange table */
struct SubTable {
	lat_int_t	min_lat;
	lat_int_t	max_lat;
	u_int32		secoffset;
};

/* Section 9 analyze table */
typedef struct {
	u_int8 record_id;					/* record id */
	char name[40];						/* record name */
	void (*func)(char *, u_int8, int);	/* analyze function */
	int len;							/* record length */
} SecRecordTab;

/*
 * macros
 */
#ifdef DEBUG
#define	ASSERT(x)	if (!(x)) { \
	fprintf(stderr, "Assertion failed: %s line %d\n", __FILE__, __LINE); \
	exit(1); \
}
#else
#define ASSERT(x)
#endif
