/*
 * scdis:  MSFS Scenery Decompiler for scasm.
 *
 * Copyright (C) 1996-2000 Takuya Murakami. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: util.c,v 1.5 2000/01/04 06:42:37 tmurakam Exp $
 */
/* util.c */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef	WIN32
#include <direct.h>
#endif
#include <errno.h>

#include "type.h"
#include "util.h"
#include "label.h"
#include "conv.h"


FILE *outfp = NULL;	/* output file pointer */
FILE *cpbat	= NULL;	/* compile batch file pointer */
FILE *sclfp = NULL;	/* sclink control file pointer */

char outdir[100];

u_int8 *bglbuf;
u_int8 *bglp;
u_int8 *bglend;

int	AnalyzePass;	/* Is 1pass or 2pass? */

/* open BGL file */
extern void
OpenBglFile(char *fname)
{
	long fsize;
	FILE *bglfp = NULL;	/* bgl file pointer */

	if (NULL == (bglfp = fopen(fname, "rb"))) {
		fprintf(stderr, "can't open file : %s\n", fname);
		exit(1);
	}

	/* get file size */
	fseek(bglfp, 0, SEEK_END);
	fsize = ftell(bglfp);
	fseek(bglfp, 0, SEEK_SET);

	/* allocate memory */
	bglbuf = malloc(fsize);
	if (!bglbuf) {
		fprintf(stderr, "no memory.\n");
		exit(1);
	}

	/* read file */
	fread(bglbuf, fsize, 1, bglfp);
	fclose(bglfp);

	bglp = bglbuf;
	bglend = bglbuf + fsize;;

	ClearLabel();
}

/* close BGL file */
extern void
CloseBglFile(void)
{
}

/* setup output directory */
extern void
SetOutDir(char *dir)
{
	int len;

	if (dir == NULL) {
		strcpy(outdir, "");
	}

	else {
		strcpy(outdir, dir);

		/* make output directory */
#ifdef	WIN32
		_mkdir(outdir);
#else	/* UNIX */
		mkdir(outdir, 0755);
#endif
		
		/* add slash to tail */
		len = strlen(outdir);
		if (outdir[len - 1] != PATH_DELIMITER) {
			outdir[len] = PATH_DELIMITER;
			outdir[len + 1] = '\0';
		}

	}
}
			
/* open output file */
extern void
OpenOutFile(char *fname, int flag)
{
	char filename[100];

	strcpy(filename, outdir);
	strcat(filename, fname);
	if (flag) {
		strcat(filename, SCMEXT);
	}

	if (NULL == (outfp = fopen(filename, "w"))) {
		fprintf(stderr, "can't open file : %s\n", filename);
		exit(1);
	}

	if (flag) {
		fprintf(sclfp, "%s%s\n", fname, BGLEXT);
		fprintf(cpbat, "scasm %s%s %s%s\n", fname, SCMEXT, fname, BGLEXT);
	}

	/*Point = 0;*/
	ClearLabel();
}

/* open output file for append */
extern void
AppendOutFile(char *fname)
{
	char filename[100];

	strcpy(filename, outdir);
	strcat(filename, fname);

	if (NULL == (outfp = fopen(filename, "a"))) {
		fprintf(stderr, "can't open file : %s\n", filename);
		exit(1);
	}

	/*Point = 0;*/
	ClearLabel();
}

/* close output file */
extern void
CloseOutFile(void)
{
	fclose(outfp);
	outfp == NULL;
}

/* open sclink control */
extern void
OpenSclinkControlFile(void)
{
	char filename[100];

	/* sclink control file */
	strcpy(filename, outdir);
	strcat(filename, SCLINKFILE);

	if (NULL == (sclfp = fopen(filename, "w"))) {
		fprintf(stderr, "can't open file : %s\n", filename);
		exit(1);
	}

	fprintf(sclfp, "; sclink control file genereted by scdis\n");
	fprintf(sclfp, "output.bgl\t; output file name\n");

	/* compile bat file */
	strcpy(filename, outdir);
	strcat(filename, COMPILEBAT);

	if (NULL == (cpbat = fopen(filename, "w"))) {
		fprintf(stderr, "can't open file : %s\n", filename);
		exit(1);
	}

	fprintf(cpbat, "rem - compile batch file genereted by scdis\n");
}

/* close sclink control */
extern void
CloseSclinkControlFile(void)
{
	fclose(sclfp);
	fclose(cpbat);
	sclfp = cpbat = NULL;
}

/* get current file pointer */
extern u_int32
GetPointer(void)
{
	return bglp - bglbuf;
}

/* move file pointer */
extern void
MovePointer(u_int32 p)
{
	bglp = bglbuf + p;
	if (bglp > bglend) {
		bglp = bglend;
	}
}

/* proceed pointer */
extern void
ProcPointer(s_int32 p)
{
	bglp += p;
	if (bglp > bglend) {
		bglp = bglend;
	}
}

/* read 1byte */
extern u_int8
get1(void)
{
	u_int8 val;

	if (IS_PASS2) {
		CheckLabel();
	}

	if (bglp < bglend) {
		val = *bglp++;
	} else {
		if (IS_PASS2) {
			fprintf(outfp, "EOF!!\n");
		}
		val = 0;
	}
	
	return val;
}

/* read 2bytes */
extern u_int16
get2(void)
{
	u_int16 val;

	if (IS_PASS2) {
		CheckLabel();
	}

	if (bglp + 1 <= bglend) {
		val = (u_int16)(*bglp++);
		val |= (u_int16)(*bglp++) << 8;
	} else {
		if (IS_PASS2) { 
			fprintf(outfp, "EOF!!!\n");
		}
		bglp = bglend;
		val = 0;
	}

	return val;
}

/* read 3 bytes */
extern u_int32
get3(void)
{
	u_int32 val;

	if (bglp + 3 <= bglend) {
		val = (u_int32)*bglp++;
		val |= (u_int32)(*bglp++) << 8;
		val |= (u_int32)(*bglp++) << 16;
	} else {
		if (IS_PASS2) { 
			fprintf(outfp, "EOF!!!\n");
		}
		bglp = bglend;
		val = 0;
	}

	return val;
}

/* read 4 bytes */
extern u_int32
get4(void)
{
	u_int32 val;

	if (IS_PASS2) {
		CheckLabel();
	}

	if (bglp + 4 <= bglend) {
		val = (u_int32)*bglp++;
		val |= (u_int32)(*bglp++) << 8;
		val |= (u_int32)(*bglp++) << 16;
		val |= (u_int32)(*bglp++) << 24;
	} else {
		if (IS_PASS2) { 
			fprintf(outfp, "EOF!!!\n");
		}
		bglp = bglend;
		val = 0;
	}

	return val;
}

/* read some length bytes */
extern void
getlong(char *buf, u_int32 len)
{
	if (IS_PASS2) {
		CheckLabel();
	}

	if (bglp + len <= bglend) {
		memcpy(buf, bglp, len);
		bglp += len;
	} else {
		bglp = bglend;
		if (IS_PASS2) { 
			fprintf(outfp, "EOF!!!\n");
		}
	}
}

/* dump to specified pointer */
extern void
DumpTo(u_int32 p)
{
	u_int32 curpoint = GetPointer();

	if (p == curpoint) return;

	if (p < curpoint) {
		fprintf(stderr, "DumpTo: invalid pointer (now:0x%x to:0x%x)\n", curpoint, p);
		exit(1);
	}

	Dump(p - curpoint);
}

/* dump specified length */
extern void
Dump(u_int32 len)
{
	u_int32 i;
	
	for (i=0; i<len; i++) {
		if (bglp >= bglend) break;
		if (IS_PASS2) {
			fprintf(outfp, "0x%x ", *bglp);
		}
		bglp++;
	}

	if (IS_PASS2) {
		fprintf(outfp, "\n");
	}
}

/* dump remain all */
extern void
DumpRemain(void)
{
	if (!IS_PASS2) {
		bglp = bglend;
		return;
	}

	while (bglp < bglend) {
		fprintf(outfp, "0x%x ", *bglp++);
	}
	fprintf(outfp, "\n;;; EOF\n");
}

/* Remove tail space characters */
extern void
RemoveTailSpace(char *str, int len)
{
	int i;
	
	str[len] = '\0';

	for (i = len - 1; i >= 0; i-- ) {
		if (str[i] == ' ') {
			str[i] = '\0';
		} else {
			break;
		}
	}
}

/* String normalize */
extern void
NormalString(char *str)
{
	char *p = str;

	while (*p) {
		switch (*p) {
		case ' ':
			*p = '_';
			break;

		case '(':
			*p = '{';
			break;

		case ')':
			*p = '}';
			break;
		}
		p++;
	}
	p--;
	while (p > str && *p == '_') {
		*p = 0;
		p--;
	}
}

/* Analyze LatRange */
#define MAXSUBTABLE	100
struct SubTable	subtable[MAXSUBTABLE];

static int
AnalyzeSubTable(struct SubTable **table, u_int8 header_byte)
{
	int i = 0;
	u_int8 c;

	while (1) {
		c = get1();
		if (c == header_byte) {
			subtable[i].min_lat = (u_int32)get2() << 8;
			subtable[i].max_lat = (u_int32)get2() << 8;
			subtable[i].secoffset = get4();

			if (i >= MAXSUBTABLE - 1) {
				fprintf(stderr, "subtable overflow.\n");
				exit(1);
			}
			i++;
		} else if (c == 0x0) {
			//printf("End of table:\n");
			break;
		}
	}
	*table = subtable;
	return i;
}

extern void
AnalyzeLatRangeTable(int secno, u_int32 sec_pointer, void (*func)(u_int32))
{
	struct SubTable	*LatRange;	/* LatRange table */
	int nument, i;
	char scmfilename[100];
	u_int32 pointer;

	/* analyze LatTable */
	nument = AnalyzeSubTable(&LatRange, 0x15);

	/* analyze ... */
	for (i=0; i<nument; i++) {
		fprintf(stderr, "\tLatRange %d\n", i + 1);

		/* open scm file */
		sprintf(scmfilename, "%s%d-%d", SECFILENAME_PRE, secno, i+1);
		OpenOutFile(scmfilename, TRUE);
		if (!outfp) {
			fprintf(stderr, "can't open file : %s\n", scmfilename);
			exit(1);
		}

		WriteHeader(secno);
			
		fprintf(outfp, "LatRange( ");
		fprintf(outfp, "%s ", latstr(lat_int2lat(LatRange[i].min_lat), NULL));
		fprintf(outfp, "%s )\n", latstr(lat_int2lat(LatRange[i].max_lat), NULL));

		pointer = sec_pointer + LatRange[i].secoffset;
		MovePointer(pointer);

		(*func)(pointer);
		CloseOutFile();
	}
}

extern void
WriteHeader(int secno)
{
	char max_north[LLBLEN], min_north[LLBLEN], max_east[LLBLEN], min_east[LLBLEN];
	char *desc;
	
	latstr(lat_long2lat(Header.max_north), max_north);
	latstr(lat_long2lat(Header.min_north), min_north);
	lonstr(lon_dec2lon(Header.max_east),   max_east);
	lonstr(lon_dec2lon(Header.min_east),   min_east);


	switch (secno) {
	case 0:	/* Section 0: VOR, ILS */
		desc = "VOR, ILS";
		break;

		/* Section 1-6: Synthetic Scenery */
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
		desc = "Synthetic scenery";
		break;
		
		/* Section 9: Procedural Scenery */
	case 9:
		desc = "Procedural scenery";
		break;

		/* Section 11: Airport */
	case 11:
		desc = "Airport";
		break;

		/* Section 13: ATIS */
	case 13:
		desc = "ATIS";
		break;

		/* Section 14: NDB */
	case 14:
		desc = "NDB";
		break;

		/* sectoin 15: Dynamic scenery */
	case 15:
		desc = "Dynamic scenery";
		break;

		/* sectoin 16: IMO Maker, Landme */
	case 16:
		desc = "IMO Marker, Landme";
		break;

		/* section 19: Exception */
	case 19:
		desc = "Exception";
		break;

	default:
		desc = "unknown";
	}

	fprintf(outfp, ";;; Disassembled by scdis %s\n", VERSION);
	fprintf(outfp, ";;; section %d - %s\n", secno, desc);
	fprintf(outfp, "Header( 1 %s %s %s %s )\n", 
		max_north, min_north, max_east, min_east);
}
